@@ -107,10 +107,10 @@ MinorKey::~MinorKey()
   _columnKey = 0;
 }
 
-void MinorKey::print() const
-{
-  cout << this->toString();
-}
+//void MinorKey::print() const
+//{
+//  PrintS(this->toString().c_str());
+//}
 
 int MinorKey::getAbsoluteRowIndex(const int i) const
 {
@@ -300,6 +300,7 @@ int MinorKey::getNumberOfColumnBlocks() const
   return _numberOfColumnBlocks;
 }
 
+#ifndef SING_NDEBUG
 int MinorKey::getSetBits(const int a) const
 {
   int b = 0;
@@ -333,6 +334,7 @@ int MinorKey::getSetBits(const int a) const
   }
   return b;
 }
+#endif
 
 MinorKey MinorKey::getSubMinorKey (const int absoluteEraseRowIndex,
                                    const int absoluteEraseColumnIndex) const
@@ -789,7 +791,8 @@ bool MinorKey::selectNextColumns (const int k, const MinorKey& mk)
 }
 
 string MinorKey::toString() const
-{
+{ return ""; }
+/*
   string t;
   string s = "(";
   unsigned int z = 0;
@@ -823,6 +826,7 @@ string MinorKey::toString() const
   s += ")";
   return s;
 }
+*/
 
 int MinorValue::g_rankingStrategy = -1;
 
