@@ -22,6 +22,7 @@
 #include <Singular/libparse.h>
 #include <Singular/feOpt.h>
 #include <kernel/dError.h>
+#include <Singular/si_signals.h>
 
 /*****************************************************************
  *
@@ -86,7 +87,7 @@ static int heCurrentHelpBrowserIndex= -1;
  * Definition: available help browsers
  *
  *****************************************************************/
-// order is improtant -- first possible help is chosen
+// order is important -- first possible help is chosen
 // moved to LIB/help.cnf
 static heBrowser_s *heHelpBrowsers=NULL;
 
@@ -136,7 +137,7 @@ void feHelp(char *str)
     }
 #endif
 
-    char* matches = StringSetS("");
+    StringSetS("");
     int found = heReKey2Entry(idxfile, str, &hentry);
 
     // Try to match with str*
@@ -177,8 +178,10 @@ void feHelp(char *str)
     else
       Warn("No help for topic '%s'", str);
     Warn("Try one of");
+    char *matches=StringEndS();
     PrintS(matches);
-    PrintS("\n");
+    omFree(matches);
+    PrintLn();
     return;
   }
 
@@ -500,7 +503,7 @@ static BOOLEAN heKey2Entry(char* filename, char* key, heEntry hentry)
         // get chksum
         hentry->url[i] = '\0';
 
-        if (fscanf(fd, "%ld\n", &(hentry->chksum)) != 1)
+        if (si_fscanf(fd, "%ld\n", &(hentry->chksum)) != 1)
         {
           hentry->chksum = -1;
         }
@@ -628,7 +631,7 @@ static int heReKey2Entry (char* filename, char* key, heEntry hentry)
   fd = fopen(filename, "r");
   if (fd == NULL) return 0;
   memset(index_key,0,MAX_HE_ENTRY_LENGTH);
-  while (fscanf(fd, "%[^\t]\t%*[^\n]\n", index_key) == 1)
+  while (si_fscanf(fd, "%[^\t]\t%*[^\n]\n", index_key) == 1)
   {
     if ((index_key[MAX_HE_ENTRY_LENGTH-1]!='\0'))
     {
@@ -658,26 +661,43 @@ static int heReKey2Entry (char* filename, char* key, heEntry hentry)
 }
 
 // test for h being a string and print it
-static BOOLEAN hePrintHelpStr(const idhdl hh,const char *id,const char *pa)
+static void hePrintHelpStr(const idhdl hh,const char *id,const char *pa)
 {
   if ((hh!=NULL) && (IDTYP(hh)==STRING_CMD))
   {
     PrintS(IDSTRING(hh));
     PrintLn();
-    return TRUE;
   }
   else
     Print("`%s` not found in package %s\n",id,pa);
-  return TRUE;;
 }
 // try to find the help string as a loaded procedure or library
 // if found, display the help and return TRUE
 // otherwise, return FALSE
 static BOOLEAN heOnlineHelp(char* s)
 {
-  idhdl h=IDROOT->get(s,myynest);
   char *ss;
+  idhdl h;
 
+  if ((ss=strstr(s,"::"))!=NULL)
+  {
+    *ss='\0';
+    ss+=2;
+    h=ggetid(s);
+    if (h!=NULL)
+    {
+      Print("help for %s from package %s\n",ss,s);
+      char s_help[200];
+      strcpy(s_help,ss);
+      strcat(s_help,"_help");
+      idhdl hh=IDPACKAGE(h)->idroot->get(s_help,0);
+      hePrintHelpStr(hh,s_help,s);
+      return TRUE;
+    }
+    else Print("package %s not found\n",s);
+    return TRUE; /* do not search the manual */
+  }
+  h=IDROOT->get(s,myynest);
   // try help for a procedure
   if (h!=NULL)
   {
@@ -699,25 +719,10 @@ static BOOLEAN heOnlineHelp(char* s)
     else if (IDTYP(h)==PACKAGE_CMD)
     {
       idhdl hh=IDPACKAGE(h)->idroot->get("info",0);
-      return hePrintHelpStr(hh,"info",s);
+      hePrintHelpStr(hh,"info",s);
+      return TRUE;
     }
-  }
-  else if ((ss=strstr(s,"::"))!=NULL)
-  {
-    *ss='\0';
-    ss+=2;
-    h=ggetid(s);
-    if (h!=NULL)
-    {
-      Print("help for %s from package %s\n",ss,s);
-      char s_help[200];
-      strcpy(s_help,ss);
-      strcat(s_help,"_help");
-      idhdl hh=IDPACKAGE(h)->idroot->get(s_help,0);
-      return hePrintHelpStr(hh,s_help,s);
-    }
-    else Print("package %s not found\n",s);
-    return TRUE; /* do not search the manual */
+    return FALSE;
   }
 
   // try help for a library
@@ -832,7 +837,7 @@ static void heBrowserHelp(heEntry hentry)
     //  Warn("Using URL '%s'.", feResource('u', 0));
     //}
     Warn("Use 'system(\"--browser\", <browser>);' to change browser,");
-    char* browsers = StringSetS("where <browser> can be: ");
+    StringSetS("where <browser> can be: ");
     int i = 0;
     i = 0;
     while (heHelpBrowsers[i].browser != NULL)
@@ -841,12 +846,14 @@ static void heBrowserHelp(heEntry hentry)
         StringAppend("\"%s\", ", heHelpBrowsers[i].browser);
       i++;
     }
+    char *browsers=StringEndS();
     if (browsers[strlen(browsers)-2] == ',')
     {
       browsers[strlen(browsers)-2] = '.';
       browsers[strlen(browsers)-1] = '\0';
     }
     WarnS(browsers);
+    omFree(browsers);
   }
 
   heCurrentHelpBrowser->help_proc(hentry, heCurrentHelpBrowserIndex);
@@ -1044,7 +1051,7 @@ static void heGenHelp(heEntry hentry, int br)
     }
   }
   Print("running `%s`\n",sys);
-  int dummy=system(sys);
+  (void)system(sys);
 }
 
 #ifdef ix86_Win
@@ -1079,20 +1086,20 @@ static void heWinHelp(heEntry hentry)
 }
 #endif
 
-static BOOLEAN heDummyInit(int warn, int br)
+static BOOLEAN heDummyInit(int /*warn*/, int /*br*/)
 {
   return TRUE;
 }
-static void heDummyHelp(heEntry hentry, int br)
+static void heDummyHelp(heEntry /*hentry*/, int /*br*/)
 {
   Werror("No functioning help browser available.");
 }
 
-static BOOLEAN heEmacsInit(int warn, int br)
+static BOOLEAN heEmacsInit(int /*warn*/, int /*br*/)
 {
   return TRUE;
 }
-static void heEmacsHelp(heEntry hentry, int br)
+static void heEmacsHelp(heEntry hentry, int /*br*/)
 {
   WarnS("Your help command could not be executed. Use");
   Warn("C-h C-s %s",
@@ -1101,7 +1108,7 @@ static void heEmacsHelp(heEntry hentry, int br)
   Warn("information on Singular running under Emacs, type C-h m.");
 }
 static int singular_manual(char *str);
-static void heBuiltinHelp(heEntry hentry, int br)
+static void heBuiltinHelp(heEntry hentry, int /*br*/)
 {
   char* node = omStrDup(hentry != NULL && *(hentry->node) != '\0' ?
                        hentry->node : "Top");
@@ -1201,7 +1208,7 @@ static int singular_manual(char *str)
   while(!feof(index))
   {
     char* dummy=fgets(buffer, BUF_LEN, index); /* */
-    (void)sscanf(buffer, "Node:%[^\177]\177%ld\n", Index, &offset);
+    (void)si_sscanf(buffer, "Node:%[^\177]\177%ld\n", Index, &offset);
     for(p=Index; *p; p++) *p = tolow(*p);/* */
     (void)strcat(Index, " ");
     if( strstr(Index, String)!=NULL)
