@@ -665,11 +665,18 @@ static BOOLEAN jiA_QRING(leftv res, leftv a,Subexpr e)
     // delete the qr copy of quotient ideal!!!
     idDelete(&qr->qideal);
   }
-  qr->qideal = id;
+  if (idElem(id)==0)
+  {
+    qr->qideal = NULL;
+    idDelete(&id);
+    IDTYP(h)=RING_CMD;
+  }
+  else
+    qr->qideal = id;
 
   // qr is a copy of currRing with the new qideal!
   #ifdef HAVE_PLURAL
-  if(rIsPluralRing(currRing))
+  if(rIsPluralRing(currRing) &&(qr->qideal!=NULL))
   {
     if (!hasFlag(a,FLAG_TWOSTD))
     {
@@ -730,7 +737,8 @@ static BOOLEAN jiA_DEF(leftv res, leftv a, Subexpr e)
 }
 /*=================== table =================*/
 #define IPASSIGN
-#define D(A) A
+#define D(A)     A
+#define NULL_VAL NULL
 #include <Singular/table.h>
 /*=================== operations ============================*/
 /*2
@@ -764,6 +772,16 @@ static BOOLEAN jiAssign_1(leftv l, leftv r)
 
   if (lt==DEF_CMD)
   {
+    if (TEST_V_ALLWARN
+    && (rt!=RING_CMD)
+    && (rt!=QRING_CMD)
+    && (l->name!=NULL)
+    && (l->e==NULL)
+    && (iiCurrArgs==NULL) /* not in proc header */
+    )
+    {
+      Warn("use `%s` instead of `def`",Tok2Cmdname(rt));
+    }
     if (l->rtyp==IDHDL)
     {
       IDTYP((idhdl)l->data)=rt;
@@ -801,7 +819,7 @@ static BOOLEAN jiAssign_1(leftv l, leftv r)
     && (dAssign[i].res!=0)) i++;
   if (dAssign[i].res!=0)
   {
-    if (TEST_V_ALLWARN) Print("assign %s=%s\n",Tok2Cmdname(lt),Tok2Cmdname(rt));
+    if (traceit&TRACE_ASSIGN) Print("assign %s=%s\n",Tok2Cmdname(lt),Tok2Cmdname(rt));
     BOOLEAN b;
     b=dAssign[i].p(ld,r,l->e);
     if(l!=ld) /* i.e. l is IDHDL, l->data is ld */
@@ -828,7 +846,7 @@ static BOOLEAN jiAssign_1(leftv l, leftv r)
         if(!failed)
         {
           failed= dAssign[i].p(ld,rn,l->e);
-          if (TEST_V_ALLWARN)
+          if (traceit&TRACE_ASSIGN)
             Print("assign %s=%s ok? %d\n",Tok2Cmdname(lt),Tok2Cmdname(rn->rtyp),!failed);
         }
         // everything done, clean up temp. variables
@@ -1095,7 +1113,7 @@ static BOOLEAN jjA_L_INTVEC(leftv l,leftv r,intvec *iv)
   {
     if (i>=iv->length())
     {
-      if (TEST_V_ALLWARN)
+      if (traceit&TRACE_ASSIGN)
       {
         Warn("expression list length(%d) does not match intmat size(%d)",
               iv->length()+exprlist_length(hh),iv->length());
@@ -1123,8 +1141,16 @@ static BOOLEAN jjA_L_INTVEC(leftv l,leftv r,intvec *iv)
     }
     hh = hh->next;
   }
-  if (IDINTVEC((idhdl)l->data)!=NULL) delete IDINTVEC((idhdl)l->data);
-  IDINTVEC((idhdl)l->data)=iv;
+  if (l->rtyp==IDHDL)
+  {
+    if (IDINTVEC((idhdl)l->data)!=NULL) delete IDINTVEC((idhdl)l->data);
+    IDINTVEC((idhdl)l->data)=iv;
+  }
+  else
+  {
+    if (l->data!=NULL) delete ((intvec*)l->data);
+    l->data=(char*)iv;
+  }
   return FALSE;
 }
 static BOOLEAN jjA_L_BIGINTMAT(leftv l,leftv r,bigintmat *bim)
@@ -1137,7 +1163,7 @@ static BOOLEAN jjA_L_BIGINTMAT(leftv l,leftv r,bigintmat *bim)
   {
     if (i>=bim->length())
     {
-      if (TEST_V_ALLWARN)
+      if (traceit&TRACE_ASSIGN)
       {
         Warn("expression list length(%d) does not match bigintmat size(%d)",
               bim->length()+exprlist_length(hh),bim->length());
@@ -1232,7 +1258,10 @@ static BOOLEAN jiA_MATRIX_L(leftv l,leftv r)
       m->m[i]=NULL;
       h=l->next;
       l->next=NULL;
+      idhdl hh=NULL;
+      if ((l->rtyp==IDHDL)&&(l->Typ()==DEF_CMD)) hh=(idhdl)l->data;
       nok=jiAssign_1(l,&t);
+      if (hh!=NULL) { ipMoveId(hh);hh=NULL;}
       l->next=h;
       if (nok)
       {
@@ -1356,7 +1385,7 @@ static BOOLEAN jiAssign_list(leftv l, leftv r)
   BOOLEAN b;
   if (/*(ld->rtyp!=LIST_CMD)
   &&*/(ld->e==NULL)
-  &&(ld->Typ()!=r->Typ()))
+  && (ld->Typ()!=r->Typ()))
   {
     sleftv tmp;
     memset(&tmp,0,sizeof(sleftv));
@@ -1365,6 +1394,18 @@ static BOOLEAN jiAssign_list(leftv l, leftv r)
     ld->CleanUp();
     memcpy(ld,&tmp,sizeof(sleftv));
   }
+  else if ((ld->e==NULL)
+  && (ld->Typ()==r->Typ())
+  && (ld->Typ()<MAX_TOK))
+  {
+    sleftv tmp;
+    memset(&tmp,0,sizeof(sleftv));
+    tmp.rtyp=r->Typ();
+    tmp.data=(char*)idrecDataInit(r->Typ());
+    b=iiAssign(&tmp,r);
+    ld->CleanUp();
+    memcpy(ld,&tmp,sizeof(sleftv));
+  }
   else
   {
     b=iiAssign(ld,r);
@@ -1439,7 +1480,7 @@ BOOLEAN iiAssign(leftv l, leftv r)
       }
       if(like_lists)
       {
-        if (TEST_V_ALLWARN) PrintS("assign list[..]=...or similiar\n");
+        if (traceit&TRACE_ASSIGN) PrintS("assign list[..]=...or similiar\n");
         if (like_lists==1)
         {
           // check blackbox/newtype type:
@@ -1468,6 +1509,14 @@ BOOLEAN iiAssign(leftv l, leftv r)
       }
     }
     // end of handling elems of list and similar
+    if (lt>MAX_TOK)
+    {
+      blackbox *bb=getBlackboxStuff(lt);
+#ifdef BLACKBOX_DEVEL
+      Print("bb-assign: bb=%lx\n",bb);
+#endif
+      return (bb==NULL) || bb->blackbox_Assign(l,r);
+    }
     rl=r->listLength();
     if (rl==1)
     {
@@ -1623,7 +1672,7 @@ BOOLEAN iiAssign(leftv l, leftv r)
         num=olm->cols()*olm->rows();
         lm=mpNew(olm->rows(),olm->cols());
         int el;
-        if ((TEST_V_ALLWARN) && (num!=(el=exprlist_length(hh))))
+        if ((traceit&TRACE_ASSIGN) && (num!=(el=exprlist_length(hh))))
         {
           Warn("expression list length(%d) does not match matrix size(%d)",el,num);
         }
@@ -1709,7 +1758,7 @@ BOOLEAN iiAssign(leftv l, leftv r)
     case STRING_CMD:
       nok=jjA_L_STRING(l,r);
       break;
-    case DEF_CMD:
+    //case DEF_CMD:
     case LIST_CMD:
       nok=jjA_L_LIST(l,r);
       break;
