@@ -237,6 +237,7 @@ char * idrec::String()
 idhdl enterid(const char * s, int lev, int t, idhdl* root, BOOLEAN init, BOOLEAN search)
 {
   if (s==NULL) return NULL;
+  if (root==NULL) return NULL;
   idhdl h;
   s=omStrDup(s);
   if (t==PACKAGE_CMD)
@@ -419,7 +420,7 @@ void killhdl2(idhdl h, idhdl * ih, ring r)
   }
   else if ((IDTYP(h)==RING_CMD)||(IDTYP(h)==QRING_CMD))
     rKill(h);
-  else
+  else if (IDDATA(h)!=NULL)
     s_internalDelete(IDTYP(h),IDDATA(h),r);
   //  general  -------------------------------------------------------------
   // now dechain it and delete idrec
@@ -590,7 +591,7 @@ void  ipMoveId(idhdl tomove)
 
 const char * piProcinfo(procinfov pi, const char *request)
 {
-  if(pi == NULL) return "empty proc";
+  if((pi == NULL)||(pi->language==LANG_NONE)) return "empty proc";
   else if (strcmp(request, "libname")  == 0) return pi->libname;
   else if (strcmp(request, "procname") == 0) return pi->procname;
   else if (strcmp(request, "type")     == 0)
@@ -667,7 +668,7 @@ void paCleanUp(package pack)
 #endif /* HAVE_DYNAMIC_LOADING */
     }
 #endif /* HAVE_STATIC */
-    omfree((ADDRESS)pack->libname);
+    omFree((ADDRESS)pack->libname);
     memset((void *) pack, 0, sizeof(sip_package));
     pack->language=LANG_NONE;
   }
@@ -693,7 +694,7 @@ void proclevel::pop()
   //if (r==NULL) Print("set ring to NULL at lev %d(%s)\n",myynest,name);
   //::currRingHdl=this->currRingHdl;
   //if((::currRingHdl==NULL)||(IDRING(::currRingHdl)!=(::currRing)))
-  //  ::currRingHdl=rFindHdl(::currRing,NULL,NULL);
+  //  ::currRingHdl=rFindHdl(::currRing,NULL);
   //Print("restore pack=%s,1.obj=%s\n",IDID(currPackHdl),IDID(currPack->idroot));
   currPackHdl=this->cPackHdl;
   currPack=this->cPack;
