@@ -85,7 +85,7 @@ char * newstruct_String(blackbox *b, void *d)
     loop
     {
       StringAppendS(a->name);
-      char *tmp=omStrDup(StringAppendS("="));
+      StringAppendS("=");
       if ((!RingDependend(a->typ))
       || ((l->m[a->pos-1].data==(void *)currRing)
          && (currRing!=NULL)))
@@ -96,25 +96,24 @@ char * newstruct_String(blackbox *b, void *d)
         }
         else
         {
-          StringSetS("");
-          char *tmp2=omStrDup(l->m[a->pos].String());
-          StringSetS(tmp);
+          char *tmp2=l->m[a->pos].String();
           if ((strlen(tmp2)>80)||(strchr(tmp2,'\n')!=NULL))
           {
-            StringAppend("<%s>",Tok2Cmdname(l->m[a->pos].rtyp));
+            StringAppendS("<");
+            StringAppendS(Tok2Cmdname(l->m[a->pos].rtyp));
+            StringAppendS(">");
           }
           else StringAppendS(tmp2);
           omFree(tmp2);
         }
       }
       else StringAppendS("??");
-      omFree(tmp);
       if (a->next==NULL) break;
       StringAppendS("\n");
       if(errorreported) break;
       a=a->next;
     }
-    return omStrDup(StringAppendS(""));
+    return StringEndS();
   }
 }
 lists lCopy_newstruct(lists L)
@@ -125,7 +124,8 @@ lists lCopy_newstruct(lists L)
   N->Init(n+1);
   for(;n>=0;n--)
   {
-    if (RingDependend(L->m[n].rtyp))
+    if (RingDependend(L->m[n].rtyp)
+    ||((L->m[n].rtyp==LIST_CMD)&&lRingDependend((lists)L->m[n].data)))
     {
       assume((L->m[n-1].rtyp==RING_CMD) || (L->m[n-1].data==NULL));
       if(L->m[n-1].data!=NULL)
@@ -276,7 +276,6 @@ BOOLEAN newstruct_Assign(leftv l, leftv r)
       return FALSE;
     }
   }
-
   else
   {
     assume(l->Typ() > MAX_TOK);
@@ -351,7 +350,7 @@ BOOLEAN newstruct_Op2(int op, leftv res, leftv a1, leftv a2)
           }
           if (nm==NULL)
           {
-            Werror("member %s nor found", a2->name);
+            Werror("member %s not found", a2->name);
             return TRUE;
           }
           if (search_ring)
@@ -365,7 +364,8 @@ BOOLEAN newstruct_Op2(int op, leftv res, leftv a1, leftv a2)
             else Werror("ring of this member is not set and no basering found");
             return r==NULL;
           }
-          else if (RingDependend(nm->typ))
+          else if (RingDependend(nm->typ)
+          || (al->m[nm->pos].RingDependend()))
           {
             if (al->m[nm->pos].data==NULL)
             {
@@ -397,6 +397,14 @@ BOOLEAN newstruct_Op2(int op, leftv res, leftv a1, leftv a2)
               currRing->ref++;
             }
           }
+          else if ((nm->typ==DEF_CMD)||(nm->typ==LIST_CMD))
+          {
+            if (al->m[nm->pos-1].data==NULL)
+            {
+              al->m[nm->pos-1].data=(void*)currRing;
+              if (currRing!=NULL) currRing->ref++;
+            }
+          }
           Subexpr r=(Subexpr)omAlloc0Bin(sSubexpr_bin);
           r->start = nm->pos+1;
           memcpy(res,a1,sizeof(sleftv));
@@ -513,6 +521,8 @@ void *newstruct_Init(blackbox *b)
   while (nm!=NULL)
   {
     l->m[nm->pos].rtyp=nm->typ;
+    if (RingDependend(nm->typ) ||(nm->typ==DEF_CMD)||(nm->typ==LIST_CMD))
+      l->m[nm->pos-1].rtyp=RING_CMD;
     l->m[nm->pos].data=idrecDataInit(nm->typ);
     nm=nm->next;
   }
@@ -564,23 +574,57 @@ BOOLEAN newstruct_serialize(blackbox *b, void *d, si_link f)
   l.data=(void*)getBlackboxName(dd->id);
   f->m->Write(f, &l);
   lists ll=(lists)d;
-  memset(&l,0,sizeof(l));
-  l.rtyp=LIST_CMD;
-  l.data=ll;
+  int Ll=lSize(ll);
+  l.rtyp=INT_CMD;
+  l.data=(void*)(long)Ll;
   f->m->Write(f, &l);
-  return FALSE;
+  // set all entries corresponding to "real" mebers to 1 in rings
+  char *rings=(char*)omAlloc0(Ll+1);
+  newstruct_member elem=dd->member;
+  while (elem!=NULL)
+  {
+    rings[elem->pos]='\1';
+    elem=elem->next;
+  }
+  int i;
+  BOOLEAN ring_changed=FALSE;
+  ring save_ring=currRing;
+  for(i=0;i<=Ll;i++)
+  {
+    if (rings[i]=='\0') // ring entry for pos i+1
+    {
+      if (ll->m[i].data!=NULL)
+      {
+        ring_changed=TRUE;
+        f->m->SetRing(f,(ring)ll->m[i].data,TRUE);
+      }
+    }
+    f->m->Write(f,&(ll->m[i]));
+  }
+  if (ring_changed)
+    f->m->SetRing(f,save_ring,FALSE);
 }
 
 BOOLEAN newstruct_deserialize(blackbox **b, void **d, si_link f)
 {
-  // newstruct is serialiazed as a list,
+  // newstruct is serialiazed as analog to a list,
   // just read a list and take data,
   // rtyp must be set correctly (to the blackbox id) by routine calling
   // newstruct_deserialize
-  leftv l=f->m->Read(f);
+  leftv l=f->m->Read(f); // int: length of list
+  int Ll=(int)(long)(l->data);
+  omFree(l);
+  lists L=(lists)omAllocBin(slists_bin);
+  L->Init(Ll+1);
+  for(int i=0;i<=Ll;i++)
+  {
+    l=f->m->Read(f);
+    memcpy(&(L->m[i]),l,sizeof(sleftv));
+    omFree(l);
+  }
   //newstruct_desc n=(newstruct_desc)b->data;
   //TODO: check compatibility of list l->data with description in n
-  *d=l->data;
+  *d=L;
   return FALSE;
 }
 
@@ -603,7 +647,11 @@ void newstruct_Print(blackbox *b,void *d)
     hh.typ=PROC_CMD;
     hh.data.pinf=p->p;
     sl=iiMake_proc(&hh,NULL,&tmp);
-    if (!sl) iiRETURNEXPR.CleanUp();
+    if (!sl)
+    {
+      if (iiRETURNEXPR.Typ()!=NONE) Warn("ignoring return value (%s)",Tok2Cmdname(iiRETURNEXPR.Typ()));
+      iiRETURNEXPR.CleanUp();
+    }
     iiRETURNEXPR.Init();
   }
   else
@@ -662,7 +710,7 @@ static newstruct_desc scanNewstructFromString(const char *s, newstruct_desc res)
       currRingHdl=save_ring;
       return NULL;
     }
-    if (RingDependend(t))
+    if (RingDependend(t) || (t==DEF_CMD)||(t==LIST_CMD))
       res->size++;    // one additional field for the ring (before the data)
     //Print("found type %s at real-pos %d",start,res->size);
     elem=(newstruct_member)omAlloc0(sizeof(*elem));
@@ -751,15 +799,28 @@ void newstructShow(newstruct_desc d)
   elem=d->member;
   while (elem!=NULL)
   {
-    Print(">>%s<< at pos %d, type %d\n",elem->name,elem->pos,elem->typ);
+    Print(">>%s<< at pos %d, type %d (%s)\n",elem->name,elem->pos,elem->typ,Tok2Cmdname(elem->typ));
+    if (RingDependend(elem->typ)|| (elem->typ==DEF_CMD) ||(elem->typ==LIST_CMD))
+      Print(">>r_%s<< at pos %d, shadow ring\n",elem->name,elem->pos-1);
     elem=elem->next;
   }
+  newstruct_proc p=d->procs;
+  while (p!=NULL)
+  {
+    Print("op:%d(%s) with %d args -> %s\n",p->t,iiTwoOps(p->t),p->args,p->p->procname);
+    p=p->next;
+  }
 }
 
 BOOLEAN newstruct_set_proc(const char *bbname,const char *func, int args,procinfov pr)
 {
   int id=0;
   blackboxIsCmd(bbname,id);
+  if (id<MAX_TOK)
+  {
+    Werror(">>%s<< is not a newstruct type",bbname);
+    return TRUE;
+  }
   blackbox *bb=getBlackboxStuff(id);
   newstruct_desc desc=(newstruct_desc)bb->data;
   newstruct_proc p=(newstruct_proc)omAlloc(sizeof(*p));
