@@ -6,10 +6,13 @@
 #include <fcntl.h>
 #include <signal.h>
 
+#include <kernel/mod2.h>
 #include <kernel/si_gmp.h>
 
 #include <omalloc/omalloc.h>
 #include <Singular/s_buff.h>
+#include <Singular/si_signals.h>
+
 //struct s_buff_s
 //{
 //    char * buff; // buffer
@@ -23,40 +26,36 @@
 
 #define S_BUFF_LEN 4096
 
-sigset_t ssi_sigmask; // set in ssiLink.cc
-sigset_t ssi_oldmask; // set in ssiLink.cc
-
-#define SSI_BLOCK_CHLD sigprocmask(SIG_SETMASK, &ssi_sigmask, &ssi_oldmask)
-#define SSI_UNBLOCK_CHLD sigprocmask(SIG_SETMASK, &ssi_oldmask, NULL)
-
 s_buff s_open(int fd)
 {
-  SSI_BLOCK_CHLD;
   s_buff F=(s_buff)omAlloc0(sizeof(*F));
   F->fd=fd;
   F->buff=(char*)omAlloc(S_BUFF_LEN);
-  SSI_UNBLOCK_CHLD;
   return F;
 }
 
 s_buff s_open_by_name(const char *n)
 {
-  SSI_BLOCK_CHLD;
-  int fd=open(n,O_RDONLY);
-  SSI_UNBLOCK_CHLD;
+  int fd=si_open(n,O_RDONLY);
   return s_open(fd);
 }
 
+int    s_free(s_buff &F)
+{
+  if (F!=NULL)
+  {
+    omFreeSize(F->buff,S_BUFF_LEN);
+    omFreeSize(F,sizeof(*F));
+    F=NULL;
+  }
+  return 0;
+}
+
 int    s_close(s_buff &F)
 {
   if (F!=NULL)
   {
-    SSI_BLOCK_CHLD;
-    omFreeSize(F->buff,S_BUFF_LEN);
     int r=close(F->fd);
-    omFreeSize(F,sizeof(*F));
-    F=NULL;
-    SSI_UNBLOCK_CHLD;
     return r;
   }
   return 0;
@@ -72,9 +71,7 @@ int s_getc(s_buff F)
   if (F->bp>=F->end)
   {
     memset(F->buff,0,S_BUFF_LEN); /*debug*/
-    SSI_BLOCK_CHLD;
-    int r=read(F->fd,F->buff,S_BUFF_LEN);
-    SSI_UNBLOCK_CHLD;
+    int r=si_read(F->fd,F->buff,S_BUFF_LEN);
     if (r<=0)
     {
       F->is_eof=1;
@@ -146,6 +143,34 @@ int s_readint(s_buff F)
   return r*neg;
 }
 
+long s_readlong(s_buff F)
+{
+  if (F==NULL)
+  {
+    printf("link closed");
+    return 0;
+  }
+  char c;
+  long neg=1;
+  long r=0;
+  //int digit=0;
+  do
+  {
+    c=s_getc(F);
+  } while((!F->is_eof) && (c<=' '));
+  if (c=='-') { neg=-1; c=s_getc(F); }
+  while(isdigit(c))
+  {
+    //digit++;
+    r=r*10+(c-'0');
+    c=s_getc(F);
+  }
+  s_ungetc(c,F);
+  //if (digit==0) { printf("unknown char %c(%d)\n",c,c); /*debug*/
+  //                printf("buffer:%s\np=%d,e=%d\n",F->buff,F->bp,F->end);fflush(stdout); } /*debug*/
+  return r*neg;
+}
+
 int s_readbytes(char *buff,int len, s_buff F)
 {
   if (F==NULL)
@@ -202,22 +227,17 @@ void s_readmpz_base(s_buff F, mpz_ptr a, int base)
     c=s_getc(F);
   } while((!F->is_eof) && (c<=' '));
   if (c=='-') { neg=-1; c=s_getc(F); }
+  char *str=(char*)omAlloc0(128);
+  int str_l=128;
+  int str_p=0;
   while(c>' ')
   {
-    if (isdigit(c))
+    if ((isdigit(c))
+    || ((c>='a') && (c<='z'))
+    || ((c>='A') && (c<='Z')))
     {
-      mpz_mul_ui(a,a,base);
-      mpz_add_ui(a,a,(c-'0'));
-    }
-    else if ((c>='a') && (c<='z'))
-    {
-      mpz_mul_ui(a,a,base);
-      mpz_add_ui(a,a,(c-'a'+10));
-    }
-    else if ((c>='A') && (c<='Z'))
-    {
-      mpz_mul_ui(a,a,base);
-      mpz_add_ui(a,a,(c-'A'+10));
+      str[str_p]=c;
+      str_p++;
     }
     else
     {
@@ -225,7 +245,14 @@ void s_readmpz_base(s_buff F, mpz_ptr a, int base)
       break;
     }
     c=s_getc(F);
+    if (str_p>=str_l)
+    {
+      str_l=str_l*2;
+      str=(char*)omRealloc0(str,str_l);
+    }
   }
+  mpz_set_str(a,str,base);
+  omFreeSize(str,str_l);
   if (neg==-1) mpz_neg(a,a);
 }
 int s_iseof(s_buff F)
