@@ -27,8 +27,10 @@
 #include <kernel/ideals.h>
 #include <kernel/numbers.h>
 #include <kernel/intvec.h>
+#include <Singular/cntrlc.h>
 #include <Singular/ssiLink.h>
 #include <Singular/pipeLink.h>
+#include <Singular/si_signals.h>
 
 // #ifdef HAVE_DBM
 // #ifdef ix86_Win
@@ -133,25 +135,31 @@ BOOLEAN slInit(si_link l, char *istr)
 
 void slCleanUp(si_link l)
 {
+  defer_shutdown++;
   (l->ref)--;
   if (l->ref == 0)
   {
     if (SI_LINK_OPEN_P(l))
     {
-      if (l->m->Kill != NULL) l->m->Kill(l);
-      else if (l->m->Close != NULL) l->m->Close(l);
+      if (l->m->Close != NULL) l->m->Close(l);
     }
+    if ((l->data != NULL) && (l->m->Kill != NULL)) l->m->Kill(l);
     omFree((ADDRESS)l->name);
     omFree((ADDRESS)l->mode);
     memset((void *) l, 0, sizeof(ip_link));
   }
+  defer_shutdown--;
+  if (!defer_shutdown && do_shutdown) m2_end(1);
 }
 
 void slKill(si_link l)
 {
+  defer_shutdown++;
   slCleanUp(l);
-  if (l->ref == 0)
+  if ((l!=NULL) &&(l->ref == 0))
     omFreeBin((ADDRESS)l,  ip_link_bin);
+  defer_shutdown--;
+  if (!defer_shutdown && do_shutdown) m2_end(1);
 }
 
 const char* slStatus(si_link l, const char *request)
@@ -164,7 +172,7 @@ const char* slStatus(si_link l, const char *request)
   else if (strcmp(request, "exists") ==0)
   {
     struct stat buf;
-    if (lstat(l->name,&buf)==0) return "yes";
+    if (si_lstat(l->name,&buf)==0) return "yes";
     else return "no";
   }
   else if (strcmp(request, "open") == 0)
@@ -187,6 +195,11 @@ const char* slStatus(si_link l, const char *request)
 }
 
 //--------------------------------------------------------------------------
+BOOLEAN slSetRingDummy(si_link, ring r, BOOLEAN send)
+{
+  if (currRing!=r) rChangeCurrRing(r);
+  return FALSE;
+}
 BOOLEAN slOpen(si_link l, short flag, leftv h)
 {
   BOOLEAN res = TRUE;
@@ -211,6 +224,7 @@ BOOLEAN slOpen(si_link l, short flag, leftv h)
         Werror("open: Error for link %s of type: %s, mode: %s, name: %s",
              c, l->m->type, l->mode, l->name);
     }
+    if (l->m->SetRing==NULL) l->m->SetRing=slSetRingDummy;
   }
   return res;
 }
@@ -238,6 +252,7 @@ BOOLEAN slClose(si_link l)
   if(! SI_LINK_OPEN_P(l))
     return FALSE;
 
+  defer_shutdown++;
   BOOLEAN res = TRUE;
   if (l->m->Close != NULL)
   {
@@ -246,6 +261,8 @@ BOOLEAN slClose(si_link l)
       Werror("close: Error for link of type: %s, mode: %s, name: %s",
            l->m->type, l->mode, l->name);
   }
+  defer_shutdown--;
+  if (!defer_shutdown && do_shutdown) m2_end(1);
   return res;
 }
 
@@ -799,8 +816,9 @@ static int DumpRhs(FILE *fd, idhdl h)
     {
       StringSetS("");
       nWrite(IDRING(h)->minpoly);
-      rhs = StringAppendS("");
+      rhs = StringEndS();
       if (fprintf(fd, "; minpoly = %s", rhs) == EOF) return EOF;
+      omFree(rhs);
     }
     else if (need_klammer) fprintf(fd, ")");
   }
@@ -845,6 +863,8 @@ BOOLEAN slGetDumpAscii(si_link l)
 
 #include <Singular/slInit.h>
 
+si_link_extension slInitTypedExtension(si_link_extension s);
+
 static si_link_extension slTypeInit(si_link_extension s, const char* type)
 {
   assume(s != NULL);
@@ -869,6 +889,10 @@ static si_link_extension slTypeInit(si_link_extension s, const char* type)
 #if 1
   else if (strcmp(type, "|") == 0)
     s->next = slInitPipeExtension(ns);
+#endif
+#if 1
+  else if (strcmp(type, "typed") == 0)
+    s->next = slInitTypedExtension(ns);
 #endif
   else
   {
@@ -904,3 +928,44 @@ void slStandardInit()
   s = si_link_root;
   s->next = NULL;
 }
+
+BOOLEAN typeWriteAscii(si_link l, leftv v)
+{
+  FILE *outfile=(FILE *)l->data;
+  BOOLEAN err=FALSE;
+  while (v!=NULL)
+  {
+    if ((v->rtyp==IDHDL)&&(v->e==NULL))
+    {
+      DumpRhs(outfile,(idhdl)v->data);
+    }
+    else
+    {
+      idrec dummy;
+      memset(&dummy,0,sizeof(dummy));
+      dummy.typ=v->Typ();
+      dummy.data.ustring=(char*)v->Data();
+      DumpRhs(outfile,&dummy);
+    }
+    fprintf(outfile,"\n");
+    v = v->next;
+  }
+  fflush(outfile);
+  return err;
+}
+
+si_link_extension slInitTypedExtension(si_link_extension s)
+{
+  s->Open=slOpenAscii;
+  s->Close=slCloseAscii;
+  s->Kill=NULL;
+  s->Read=slReadAscii;
+  s->Read2=slReadAscii2;
+  s->Write=typeWriteAscii;
+  s->Dump=slDumpAscii;
+  s->GetDump=slGetDumpAscii;
+
+  s->Status=slStatusAscii;
+  s->type="typed";
+  return s;
+}
