@@ -366,6 +366,8 @@ BOOLEAN sleftv::RingDependend()
     return TRUE;
   if (rt==LIST_CMD)
     return lRingDependend((lists)Data());
+  if (this->next!=NULL)
+    return this->next->RingDependend();
   return FALSE;
 }
 
@@ -410,7 +412,7 @@ static inline void * s_internalCopy(const int t,  void *d)
     case QRING_CMD:
       {
         ring r=(ring)d;
-        r->ref++;
+        if (r!=NULL) r->ref++;
         //Print("+  ring %d, ref %d\n",r,r->ref);
         return d;
       }
@@ -439,6 +441,7 @@ static inline void * s_internalCopy(const int t,  void *d)
 
 void s_internalDelete(const int t,  void *d, const ring r)
 {
+  assume(d!=NULL);
   switch (t)
   {
     case INTVEC_CMD:
@@ -514,12 +517,12 @@ void s_internalDelete(const int t,  void *d, const ring r)
     case QRING_CMD:
     {
       ring R=(ring)d;
+      if ((R!=currRing)||(R->ref>=0))
+        rKill(R);
       #ifdef TEST
-      if ((R==currRing)&&(R->ref<=0))
-        Print("currRing? ref=%d\n",R->ref);
       else
+        Print("currRing? ref=%d\n",R->ref);
       #endif
-      rKill(R);
       break;
     }
     case RESOLUTION_CMD:
@@ -719,10 +722,11 @@ char *  sleftv::String(void *d, BOOLEAN typed, int dim)
             char* ps = pString((poly) d);
             s = (char*) omAlloc(strlen(ps) + 10);
             sprintf(s,"%s(%s)", (t /*Typ()*/ == POLY_CMD ? "poly" : "vector"), ps);
+	    omFree(ps);
             return s;
           }
           else
-            return omStrDup(pString((poly)d));
+            return pString((poly)d);
 
         case NUMBER_CMD:
           StringSetS((char*) (typed ? "number(" : ""));
@@ -746,16 +750,16 @@ char *  sleftv::String(void *d, BOOLEAN typed, int dim)
             nWrite(n);
             nDelete(&n);
           }
-          s = StringAppendS((char*) (typed ? ")" : ""));
-          return omStrDup(s);
+          StringAppendS((char*) (typed ? ")" : ""));
+          return StringEndS();
 
         case BIGINT_CMD:
           {
           StringSetS((char*) (typed ? "bigint(" : ""));
           number nl=(number)d;
           nlWrite(nl,NULL);
-          s = StringAppendS((char*) (typed ? ")" : ""));
-          return omStrDup(s);
+          StringAppendS((char*) (typed ? ")" : ""));
+          return StringEndS();
           }
 
         case MATRIX_CMD:
@@ -820,10 +824,11 @@ char *  sleftv::String(void *d, BOOLEAN typed, int dim)
             char* ns = (char*) omAlloc0(strlen(s) + 40);
             sprintf(ns, "bigintmat(bigintvec(%s),%d,%d)", s, bim->rows(), bim->cols());
             omCheckAddr(ns);
+            omFree(s);
             return ns;
           }
           else
-            return omStrDup(s);
+            return s;
         } 
         case RING_CMD:
         case QRING_CMD:
@@ -1041,16 +1046,16 @@ void * sleftv::Data()
         idhdl h=(idhdl)data;
         return  ((idhdl)h->data.ustring)->data.ustring;
       }
-      case VECHO:      return (void *)si_echo;
-      case VPRINTLEVEL:return (void *)printlevel;
-      case VCOLMAX:    return (void *)colmax;
-      case VTIMER:     return (void *)getTimer();
-      case VRTIMER:    return (void *)getRTimer();
-      case VOICE:      return (void *)(myynest+1);
-      case VMAXDEG:    return (void *)Kstd1_deg;
-      case VMAXMULT:   return (void *)Kstd1_mu;
-      case TRACE:      return (void *)traceit;
-      case VSHORTOUT:  return (void *)(currRing != NULL ? currRing->ShortOut : 0);
+      case VECHO:      return (void *)(long)si_echo;
+      case VPRINTLEVEL:return (void *)(long)printlevel;
+      case VCOLMAX:    return (void *)(long)colmax;
+      case VTIMER:     return (void *)(long)getTimer();
+      case VRTIMER:    return (void *)(long)getRTimer();
+      case VOICE:      return (void *)(long)(myynest+1);
+      case VMAXDEG:    return (void *)(long)Kstd1_deg;
+      case VMAXMULT:   return (void *)(long)Kstd1_mu;
+      case TRACE:      return (void *)(long)traceit;
+      case VSHORTOUT:  return (void *)(long)(currRing != NULL ? currRing->ShortOut : 0);
       case VMINPOLY:   if (currRing != NULL &&
                            (currRing->minpoly!=NULL)&&(!rField_is_GF()))
                        /* Q(a), Fp(a), but not GF(q) */
@@ -1097,7 +1102,7 @@ void * sleftv::Data()
           Werror("wrong range[%d] in intvec(%d)",index,iv->length());
       }
       else
-        r=(char *)((*iv)[index-1]);
+        r=(char *)(long)((*iv)[index-1]);
       break;
     }
     case INTMAT_CMD:
@@ -1113,7 +1118,7 @@ void * sleftv::Data()
                                                      iv->rows(),iv->cols());
       }
       else
-        r=(char *)(IMATELEM((*iv),index,e->next->start));
+        r=(char *)(long)(IMATELEM((*iv),index,e->next->start));
       break;
     }
     case BIGINTMAT_CMD:
@@ -1257,7 +1262,9 @@ attr * sleftv::Attribute()
 {
   if (e==NULL) return &attribute;
   if ((rtyp==LIST_CMD)
-  ||((rtyp==IDHDL)&&(IDTYP((idhdl)data)==LIST_CMD)))
+  ||((rtyp==IDHDL)&&(IDTYP((idhdl)data)==LIST_CMD))
+  || (rtyp>MAX_TOK)
+  || ((rtyp==IDHDL)&&(IDTYP((idhdl)data)>MAX_TOK)))
   {
     leftv v=LData();
     return &(v->attribute);
@@ -1270,11 +1277,18 @@ leftv sleftv::LData()
   if (e!=NULL)
   {
     lists l=NULL;
+    blackbox *b=getBlackboxStuff(rtyp);
 
-    if (rtyp==LIST_CMD)
+    if ((rtyp==LIST_CMD)
+    || ((b!=NULL)&&(BB_LIKE_LIST(b))))
       l=(lists)data;
     else if ((rtyp==IDHDL)&& (IDTYP((idhdl)data)==LIST_CMD))
       l=IDLIST((idhdl)data);
+    else if ((rtyp==IDHDL)&& (IDTYP((idhdl)data)>MAX_TOK))
+    {
+      b=getBlackboxStuff(IDTYP((idhdl)data));
+      if (BB_LIKE_LIST(b)) l=IDLIST((idhdl)data);
+    }
     else if (rtyp==ALIAS_CMD)
     {
       idhdl h=(idhdl)data;
@@ -1297,6 +1311,7 @@ leftv sleftv::LData()
   return this;
 }
 
+#if 0
 leftv sleftv::LHdl()
 {
   if (e!=NULL)
@@ -1323,6 +1338,7 @@ leftv sleftv::LHdl()
   }
   return this;
 }
+#endif
 
 BOOLEAN assumeStdFlag(leftv h)
 {
@@ -1355,10 +1371,10 @@ void syMake(leftv v,const char * id, idhdl packhdl)
   * 1) reserved id: done by scanner
   * 2) `basering` / 'Current`
   * 3) existing identifier, local
-  * 4) ringvar, local ring
+  * 4) ringvar, ringpar, local ring
   * 5) existing identifier, global
   * 6) monom (resp. number), local ring: consisting of:
-  * 6') ringvar, global ring
+  * 6') ringvar, ringpar, global ring
   * 6'') monom (resp. number), local ring
   * 7) monom (resp. number), non-local ring
   * 8) basering
@@ -1448,6 +1464,21 @@ void syMake(leftv v,const char * id, idhdl packhdl)
         v->rtyp = POLY_CMD;
         return;
       }
+      if ((currRing->algring!=NULL) && ((vnr=r_IsRingVar(id,currRing->algring))>=0))
+      {
+	BOOLEAN ok=FALSE;
+	poly p = pmInit(id,ok);
+	if (ok && (p!=NULL))
+	{
+          v->data = (void *)pGetCoeff(p);
+	  v->rtyp = NUMBER_CMD;
+          v->name = id;
+          pGetCoeff(p)=NULL;
+          pLmFree(p);
+          return;
+	}
+      }
+
     }
     /* 5. existing identifier, global */
     if (h!=NULL)
@@ -1488,6 +1519,14 @@ void syMake(leftv v,const char * id, idhdl packhdl)
           v->rtyp = POLY_CMD;
           v->name = id;
         }
+        if (TEST_V_ALLWARN /*&& (myynest>0)*/
+        && ((rIsRingVar(id)>=0)
+          || ((currRing->P>0)
+             &&(r_IsRingVar(id, currRing->algring)>=0))))
+        {
+        // WARNING: do not use ring variable names in procedures
+          Warn("use of variable >>%s<< in a procedure in line %s",id,my_yylinebuf);
+        }
         return;
       }
     }
@@ -1679,16 +1718,23 @@ int sleftv::Eval()
         {
           if (d->argc>=1) nok=d->arg1.Eval();
           if ((!nok) && (d->argc>=2))
-          { nok=d->arg2.Eval(); d->arg1.next=&d->arg2; }
+          {
+            nok=d->arg2.Eval();
+            d->arg1.next=(leftv)omAllocBin(sleftv_bin);
+            memcpy(d->arg1.next,&d->arg2,sizeof(sleftv));
+            d->arg2.Init();
+          }
           if ((!nok) && (d->argc==3))
-          { nok=d->arg3.Eval(); d->arg2.next=&d->arg3; }
+          {
+            nok=d->arg3.Eval();
+            d->arg1.next->next=(leftv)omAllocBin(sleftv_bin);
+            memcpy(d->arg1.next->next,&d->arg3,sizeof(sleftv));
+            d->arg3.Init();
+          }
           if (d->argc==0)
             nok=nok||iiExprArithM(this,NULL,d->op);
           else
             nok=nok||iiExprArithM(this,&d->arg1,d->op);
-          d->arg1.next=NULL;
-          d->arg2.next=NULL;
-          d->arg3.next=NULL;
         }
         else
         {
@@ -1729,6 +1775,8 @@ int sleftv::Eval()
     &&(name!=NULL))
   {
      syMake(this,name);
+     //if (rtyp==0) /*not found */
+     //  nok=TRUE;
   }
 #ifdef MDEBUG
   switch(Typ())
