@@ -7,6 +7,7 @@
 #include <omalloc/omalloc.h>
 #include <kernel/febase.h>
 #include <kernel/longrat.h>
+#include <kernel/polys.h>
 #include <Singular/subexpr.h>
 #include <Singular/ipshell.h>
 #include <Singular/lists.h>
@@ -40,7 +41,7 @@ char* bbfan_String(blackbox *b, void *d)
   else
   {
     gfan::ZFan* zf = (gfan::ZFan*)d;
-    std::string s = zf->toString();
+    std::string s = zf->toStringJustRaysAndMaximalCones();
     return omStrDup(s.c_str());
   }
 }
@@ -120,7 +121,7 @@ static gfan::IntMatrix permutationIntMatrix(const bigintmat* iv)
       number temp1 = nlInit(1,NULL);
       number temp2 = nlSub(IMATELEM(*iv, r, c),temp1);
       ivCopy->set(r,c,temp2);
-      nlDelete(&temp1,NULL); 
+      nlDelete(&temp1,NULL);
       nlDelete(&temp2,NULL);
     }
   gfan::ZMatrix* zm = bigintmatToZMatrix(ivCopy);
@@ -265,8 +266,8 @@ BOOLEAN numberOfConesOfDimension(leftv res, leftv args)
           int d = (int)(long)v->Data();
           int o = (int)(long)w->Data();
           int m = (int)(long)x->Data();
-          if ( (0<=d) && (d <= zf->getAmbientDimension()) 
-                      && ((o == 0) || (o == 1)) 
+          if ( (0<=d) && (d <= zf->getAmbientDimension())
+                      && ((o == 0) || (o == 1))
                       && ((m == 0) || (m == 1)))
           {
             bool oo = (bool) o;
@@ -398,7 +399,7 @@ BOOLEAN insertCone(leftv res, leftv args)
       if (n != 0)
       {
         if (!isCompatible(zf,zc))
-        {          
+        {
           WerrorS("insertCone: cone and fan not compatible");
           return TRUE;
         }
@@ -508,7 +509,7 @@ BOOLEAN removeCone(leftv res, leftv args)
       if (n != 0)
       {
         if (!containsInCollection(zf,zc))
-        {          
+        {
           WerrorS("removeCone: cone not contained in fan");
           return TRUE;
         }
@@ -725,7 +726,7 @@ BOOLEAN fVector(leftv res, leftv args)
   return TRUE;
 }
 
-gfan::ZMatrix rays(gfan::ZFan* zf)
+gfan::ZMatrix rays(const gfan::ZFan* const zf)
 {
   int linDim = zf->getLinealityDimension();
   gfan::ZMatrix rays(0,zf->getAmbientDimension());
@@ -747,7 +748,7 @@ int numberOfConesWithVector(gfan::ZFan* zf, gfan::ZVector* v)
     if (zc.contains(*v))
     {
       count = count +1;
-      if (count > 1) 
+      if (count > 1)
         return count;
     }
   }
@@ -770,7 +771,7 @@ BOOLEAN numberOfConesWithVector(leftv res, leftv args)
         WerrorS("numberOfConesWithVector: mismatching dimensions");
         return TRUE;
       }
-      gfan::ZVector* v1 = bigintmatToZVector(*v0); 
+      gfan::ZVector* v1 = bigintmatToZVector(*v0);
       int count = numberOfConesWithVector(zf, v1);
       delete v1;
       res->rtyp = INT_CMD;
@@ -872,53 +873,110 @@ BOOLEAN fanViaCones(leftv res, leftv args)
   return TRUE;
 }
 
-lists listOfFacets(const gfan::ZCone &zc)
-{
-  gfan::ZMatrix inequalities = zc.getFacets();
-  gfan::ZMatrix equations = zc.getImpliedEquations();
-  lists L = (lists)omAllocBin(slists_bin);
-  int r = inequalities.getHeight();
-  int c = inequalities.getWidth();
-  L->Init(r);
-  
-  /* next we iterate over each of the r facets, build the respective cone and add it to the list */
-  /* this is the i=0 case */
-  gfan::ZMatrix newInequalities = inequalities.submatrix(1,0,r,c);
-  gfan::ZMatrix newEquations = equations; 
-  newEquations.appendRow(inequalities[0]);
-  L->m[0].rtyp = coneID; L->m[0].data=(void*) new gfan::ZCone(newInequalities,newEquations);
-  
-  /* these are the cases i=1,...,r-2 */
-  for (int i=1; i<r-1; i++)
-  {
-    newInequalities = inequalities.submatrix(0,0,i-1,c);
-    newInequalities.append(inequalities.submatrix(i+1,0,r,c));
-    newEquations = equations;
-    newEquations.appendRow(inequalities[i]);
-    L->m[i].rtyp = coneID; L->m[i].data=(void*) new gfan::ZCone(newInequalities,newEquations);
-  }
-  
-  /* this is the i=r-1 case */
-  newInequalities = inequalities.submatrix(0,0,r-1,c);
-  newEquations = equations;
-  newEquations.appendRow(inequalities[r]);
-  L->m[r-1].rtyp = coneID; L->m[r-1].data=(void*) new gfan::ZCone(newInequalities,newEquations);
-
-  return L;
-}
-
-BOOLEAN listOfFacets(leftv res, leftv args)
+BOOLEAN tropicalVariety(leftv res, leftv args)
 {
   leftv u=args;
-  if ((u != NULL) && (u->Typ() == coneID))
+  if ((u != NULL) && (u->Typ() == POLY_CMD))
   {
-    gfan::ZCone* zc = (gfan::ZCone*) u->Data();
-    lists L = listOfFacets(*zc);
-    res->rtyp = LIST_CMD;
-    res->data = (void*) L;
+    int n = rVar(currRing);
+    gfan::ZFan* zf = new gfan::ZFan(n);
+    int* expv1 = (int*)omAlloc((n+1)*sizeof(int));
+    int* expv2 = (int*)omAlloc((n+1)*sizeof(int));
+    int* expvr = (int*)omAlloc((n+1)*sizeof(int));
+    gfan::ZVector expw1 = gfan::ZVector(n);
+    gfan::ZVector expw2 = gfan::ZVector(n);
+    gfan::ZVector expwr = gfan::ZVector(n);
+    gfan::ZMatrix eq, ineq;
+    for (poly s1=(poly)u->Data(); s1!=NULL; pIter(s1))
+    {
+      pGetExpV(s1,expv1);
+      expw1 = intStar2ZVector(n,expv1);
+      for (poly s2=pNext(s1); s2!=NULL; pIter(s2))
+      {
+        pGetExpV(s2,expv2);
+        expw2 = intStar2ZVector(n,expv2);
+        eq = gfan::ZMatrix(0,n);
+        eq.appendRow(expw1-expw2);
+        ineq = gfan::ZMatrix(0,n);
+        for (poly r=(poly)u->Data(); r!=NULL; pIter(r))
+        {
+          pGetExpV(r,expvr);
+          expwr = intStar2ZVector(n,expvr);
+          if ((r!=s1) && (r!=s2))
+          {
+            ineq.appendRow(expw1-expwr);
+          }
+        }
+        gfan::ZCone zc = gfan::ZCone(ineq,eq);
+        zf->insert(zc);
+      }
+    }
+    omFreeSize(expv1,(n+1)*sizeof(int));
+    omFreeSize(expv2,(n+1)*sizeof(int));
+    omFreeSize(expvr,(n+1)*sizeof(int));
+    res->rtyp = fanID;
+    res->data = (void*) zf;
     return FALSE;
   }
-  WerrorS("listOfFacets: unexpected parameters");
+  WerrorS("tropicalVariety: unexpected parameters");
+  return TRUE;
+}
+
+gfan::ZFan* commonRefinement(gfan::ZFan* zf, gfan::ZFan* zg)
+{
+  assume(zf->getAmbientDimension() == zg->getAmbientDimension());
+
+  // gather all maximal cones of f and g
+  std::list<gfan::ZCone> maximalConesOfF;
+  for (int d=0; d<=zf->getAmbientDimension(); d++)
+  {
+    for (int i=0; i<zf->numberOfConesOfDimension(d,0,1); i++)
+    {
+      maximalConesOfF.push_back(zf->getCone(d,i,0,1));
+    }
+  }
+
+  std::list<gfan::ZCone> maximalConesOfG;
+  for (int d=0; d<=zg->getAmbientDimension(); d++)
+  {
+    for (int i=0; i<zg->numberOfConesOfDimension(d,0,1); i++)
+    {
+      maximalConesOfG.push_back(zg->getCone(d,i,0,1));
+    }
+  }
+
+  // construct a new fan out of their intersections
+  gfan::ZFan* zr = new gfan::ZFan(zf->getAmbientDimension());
+  for (std::list<gfan::ZCone>::iterator itf=maximalConesOfF.begin();
+       itf != maximalConesOfF.end(); itf++)
+  {
+    for (std::list<gfan::ZCone>::iterator itg=maximalConesOfG.begin();
+         itg != maximalConesOfG.end(); itg++)
+    {
+      zr->insert(intersection(*itf,*itg));
+    }
+  }
+
+  return zr;
+}
+
+BOOLEAN commonRefinement(leftv res, leftv args)
+{
+  leftv u=args;
+  if ((u != NULL) && (u->Typ() == fanID))
+  {
+    leftv v=u->next;
+    if ((v != NULL) && (v->Typ() == fanID))
+    {
+      gfan::ZFan* zf = (gfan::ZFan*) u->Data();
+      gfan::ZFan* zg = (gfan::ZFan*) v->Data();
+      gfan::ZFan* zr = commonRefinement(zf,zg);
+      res->rtyp = fanID;
+      res->data = (void*) zr;
+      return FALSE;
+    }
+  }
+  WerrorS("commonRefinement: unexpected parameters");
   return TRUE;
 }
 
@@ -1004,6 +1062,8 @@ void bbfan_setup()
   iiAddCproc("","fanFromString",FALSE,fanFromString);
   iiAddCproc("","fanViaCones",FALSE,fanViaCones);
   iiAddCproc("","numberOfConesWithVector",FALSE,numberOfConesWithVector);
+  iiAddCproc("","tropicalVariety",FALSE,tropicalVariety);
+  iiAddCproc("","commonRefinement",FALSE,commonRefinement);
   // iiAddCproc("","isComplete",FALSE,isComplete);  not working as expected, should leave this to polymake
   iiAddCproc("","fVector",FALSE,fVector);
   iiAddCproc("","containsInCollection",FALSE,containsInCollection);
