@@ -7,7 +7,7 @@
 #include <polymake/Set.h>
 #include <polymake/common/lattice_tools.h>
 // #include <polymake/perl/macros.h>
-// #include <polymake/IncidenceMatrix.h>
+#include <polymake/IncidenceMatrix.h>
 
 #include <gfanlib/gfanlib.h>
 #include <gfanlib/gfanlib_q.h>
@@ -16,19 +16,22 @@
 // #include <kernel/structs.h>
 // #include <kernel/febase.h>
 #include <kernel/intvec.h>
+// #include <kernel/numbers.h>
+#include <kernel/bigintmat.h>
 
 // #include <callgfanlib/bbcone.h>
 // #include <callgfanlib/bbfan.h>
 // #include <callgfanlib/bbpolytope.h>
 
 #include <Singular/mod2.h>
+#include <Singular/lists.h>
 // #include <Singular/blackbox.h>
 // #include <Singular/ipshell.h>
 // #include <Singular/subexpr.h>
 // #include <Singular/tok.h>
 
 
-/* Functions for converting Integers, Rationals and their Matrices 
+/* Functions for converting Integers, Rationals and their Matrices
    in between C++, gfan, polymake and singular */
 
 /* gfan -> polymake */
@@ -51,7 +54,7 @@ polymake::Rational GfRational2PmRational (const gfan::Rational& gr)
 
 polymake::Vector<polymake::Integer> Intvec2PmVectorInteger (const intvec* iv)
 {
-  polymake::Vector<polymake::Integer> vi(iv->length()); 
+  polymake::Vector<polymake::Integer> vi(iv->length());
   for(int i=1; i<=iv->length(); i++)
   {
     vi[i-1]=(*iv)[i-1];
@@ -87,7 +90,20 @@ gfan::Integer PmInteger2GfInteger (const polymake::Integer& pi)
 {
   mpz_class cache(pi.get_rep());
   gfan::Integer gi(cache.get_mpz_t());
-  return gi;  
+  return gi;
+}
+
+number PmInteger2Number (const polymake::Integer& pi)
+{
+  mpz_class cache(pi.get_rep());
+  long m = 268435456;
+  if(mpz_cmp_si(cache.get_mpz_t(),m))
+  {
+    int temp = (int) mpz_get_si(cache.get_mpz_t());
+    return nlInit(temp,NULL);
+  }
+  else
+    return nlInitgmp(cache.get_mpz_t());
 }
 
 gfan::Rational PmRational2GfRational (const polymake::Rational& pr)
@@ -125,8 +141,8 @@ int PmInteger2Int(const polymake::Integer& pi, bool &ok)
 {
   int i=0;
   try
-  { 
-    i = pi.to_int(); 
+  {
+    i = pi.to_int();
   }
   catch (const std::exception& ex)
   {
@@ -152,25 +168,94 @@ intvec* PmMatrixInteger2Intvec (polymake::Matrix<polymake::Integer>* mi, bool &o
   int cols = mi->cols();
   intvec* iv = new intvec(rows,cols,0);
   const polymake::Integer* pi = concat_rows(*mi).begin();
-  for (int r = 1; r <= rows; r++) 
-    for (int c = 1; c <= cols; c++) 
+  for (int r = 1; r <= rows; r++)
+    for (int c = 1; c <= cols; c++)
     {
-      IMATELEM(*iv,r,c) = PmInteger2Int(*pi, ok); 
-      pi++; 
+      IMATELEM(*iv,r,c) = PmInteger2Int(*pi, ok);
+      pi++;
     }
-  return iv;  
+  return iv;
+}
+
+bigintmat* PmMatrixInteger2Bigintmat (polymake::Matrix<polymake::Integer>* mi)
+{
+  int rows = mi->rows();
+  int cols = mi->cols();
+  bigintmat* bim= new bigintmat(rows,cols);
+  const polymake::Integer* pi = concat_rows(*mi).begin();
+  for (int r = 1; r <= rows; r++)
+    for (int c = 1; c <= cols; c++)
+    {
+      number temp = PmInteger2Number(*pi);
+      bim->set(r,c,temp);
+      nlDelete(&temp,NULL);
+      pi++;
+    }
+  return bim;
+}
+
+lists PmIncidenceMatrix2ListOfIntvecs (polymake::IncidenceMatrix<polymake::NonSymmetric>* icmat)
+{
+  int rows = icmat->rows();
+  int cols = icmat->cols();
+  lists L = (lists)omAllocBin(slists_bin);
+  L->Init(rows);
+
+  for (int r = 0; r < rows; r++)
+  {
+    intvec* iv = new intvec(cols); int i=0;
+    for (int c = 0; c < cols; c++)
+    {
+      if ((*icmat).row(r).exists(c))
+        { (*iv)[i]=c; i++; }
+    }
+    iv->resize(i);
+    L->m[r].rtyp = INTVEC_CMD;
+    L->m[r].data = (void*) iv;
+  }
+
+  return L;
+}
+
+
+lists PmAdjacencyMatrix2ListOfEdges (polymake::IncidenceMatrix<polymake::NonSymmetric>* icmat)
+{
+  int rows = icmat->rows();
+  int cols = icmat->cols();
+
+  // counting number of edges
+  int i=0;
+  for (int r = 0; r < rows; r++)
+  {
+    for (int c = 0; c < cols; c++)
+    {
+      if ((*icmat).row(r).exists(c) && r<c)
+        i++;
+    }
+  }
+
+  lists L = (lists)omAllocBin(slists_bin);
+  L->Init(i);
+
+  i=0;
+  for (int r = 0; r < rows; r++)
+  {
+    for (int c = 0; c < cols; c++)
+    {
+      if ((*icmat).row(r).exists(c) && r<c)
+      {
+        intvec* iv = new intvec(2);
+        (*iv)[0]=r; (*iv)[1]=c;
+        L->m[i].rtyp = INTVEC_CMD;
+        L->m[i].data = (void*) iv;
+        i++;
+      }
+    }
+  }
+
+  return L;
 }
 
-// intvec* PmIncidenceMatrix2Intvec (polymake::IncidenceMatrix<polymake::NonSymmetric>* icmat)
-// {
-//   int rows = icmat->rows();
-//   int cols = icmat->cols();
-//   intvec* iv = new intvec(rows,cols,0);
-//   for (int r = 1; r <= rows; r++)
-//     for (int c = 1; c <= cols; c++)
-//       IMATELEM(*iv,r,c) = (int) (*icmat).row(r).exists(c);
-//   return iv;
-// }
 
 intvec* PmSetInteger2Intvec (polymake::Set<polymake::Integer>* si, bool &b)
 {
@@ -212,15 +297,15 @@ gfan::ZCone* PmCone2ZCone (polymake::perl::Object* pc)
     gfan::ZMatrix zv, zw, zx, zy, zz;
     // the following branching statements are to cover cases in which polymake returns empty matrices
     // by convention, gfanlib ignores empty matrices, hence zero matrices of right dimensions have to be supplied
-    if (ineqrational.cols()!=0) 
-    {  
+    if (ineqrational.cols()!=0)
+    {
       polymake::Matrix<polymake::Integer> ineqinteger = polymake::common::primitive(ineqrational);
       zv = PmMatrixInteger2GfZMatrix(&ineqinteger);
     }
     else
       zv = gfan::ZMatrix(0, ambientdim2);
     if (eqrational.cols()!=0)
-    {  
+    {
       polymake::Matrix<polymake::Integer> eqinteger = polymake::common::primitive(eqrational);
       zw = PmMatrixInteger2GfZMatrix(&eqinteger);
     }
@@ -265,7 +350,7 @@ gfan::ZCone* PmPolytope2ZPolytope (polymake::perl::Object* pp)
     // polymake::Matrix<polymake::Rational> linrational = pp->give("LINEALITY_SPACE");
 
     gfan::ZMatrix zv, zw;
-    // the following branching statements are to cover the cases when polymake returns empty matrices 
+    // the following branching statements are to cover the cases when polymake returns empty matrices
     // by convention, gfanlib ignores empty matrices, hence zero matrices of right dimensions have to be supplied
     if (ineqrational.cols()!=0)
     {
@@ -298,7 +383,7 @@ gfan::ZCone* PmPolytope2ZPolytope (polymake::perl::Object* pp)
     // else
     //   zy = gfan::ZMatrix(0, ambientdim2);
 
-    // gfan::ZCone* zp = new gfan::ZCone(zv,zw,zx,zy,zz,3);  
+    // gfan::ZCone* zp = new gfan::ZCone(zv,zw,zx,zy,zz,3);
     gfan::ZCone* zp = new gfan::ZCone(zv,zw,3);
 
     return zp;
@@ -338,7 +423,7 @@ polymake::perl::Object* ZCone2PmCone (gfan::ZCone* zc)
   gc->take("LINEAR_SPAN") << GfZMatrix2PmMatrixInteger(&equations);
 
   // if(zc->areExtremeRaysKnown())
-  //   {  
+  //   {
   //     gfan::ZMatrix extremeRays = zc->extremeRays();
   //     gc->take("RAYS") << GfZMatrix2PmMatrixInteger(&extremeRays);
   //   }
