@@ -69,6 +69,13 @@ CanonicalForm::deepCopy() const
     else
         return CanonicalForm( value->deepCopyObject() );
 }
+
+void
+CanonicalForm::mpzval(mpz_t val) const
+{
+    ASSERT (!is_imm (value) && value->levelcoeff() == IntegerDomain, "non-immediate integer expected");
+    getmpi (value, val);
+}
 //}}}
 
 //{{{ predicates
@@ -478,6 +485,12 @@ CanonicalForm::degree( const Variable & v ) const
 // are considered polynomials, not coefficients, and such may
 // have a taildegree larger than zero.
 //
+// tailcoeff( v ) returns the tail coefficient of CO where CO is
+// considered an univariate polynomial in the polynomial variable
+// v.
+// Note: If v is less than the main variable of CO we have to
+// swap variables which may be quite expensive.
+//
 // See also: InternalCF::tailcoeff(), InternalCF::tailcoeff(),
 // InternalPoly::tailcoeff(), InternalPoly::taildegree,
 // ::tailcoeff(), ::taildegree()
@@ -492,6 +505,27 @@ CanonicalForm::tailcoeff () const
         return value->tailcoeff();
 }
 
+CanonicalForm
+CanonicalForm::tailcoeff (const Variable& v) const
+{
+    if ( is_imm( value ) || value->inCoeffDomain() )
+        return *this;
+
+    Variable x = value->variable();
+    if ( v > x )
+        return *this;
+    else if ( v == x )
+        return value->tailcoeff();
+    else {
+        CanonicalForm f = swapvar( *this, v, x );
+         if ( f.mvar() == x )
+             return swapvar( f.value->tailcoeff(), v, x );
+         else
+            // v did not occur in f
+            return *this;
+    }
+}
+
 int
 CanonicalForm::taildegree () const
 {
@@ -1765,13 +1799,7 @@ operator >> ( ISTREAM & is, CanonicalForm & cf )
 #endif /* NOSTREAMIO */
 //}}}
 
-//{{{ genCoeff(), genOne(), genZero()
-CanonicalForm
-CanonicalForm::genCoeff( int type, int i )
-{
-    return CanonicalForm( CFFactory::basic( type, i ) );
-}
-
+//{{{ genOne(), genZero()
 CanonicalForm
 CanonicalForm::genZero() const
 {
