@@ -17,11 +17,14 @@
 #endif
 #endif /* NOSTREAMIO */
 
+#include <stdint.h>
+
 #include "cf_defs.h"
 #include "variable.h"
 #include "cf_gmp.h"
 #include "templates/ftmpl_list.h"
 #include "templates/ftmpl_array.h"
+#include "templates/ftmpl_afactor.h"
 #include "templates/ftmpl_factor.h"
 #include "templates/ftmpl_matrix.h"
 
@@ -49,7 +52,7 @@ class InternalCF;
 inline int is_imm ( const InternalCF * const ptr )
 {
     // returns 0 if ptr is not immediate
-    return ( ((int)((long)ptr)) & 3 );
+    return ( ((int)((intptr_t)ptr)) & 3 );
 }
 
 
@@ -359,6 +362,9 @@ bool isOn( int );
 //}}}
 
 //{{{ type definitions
+typedef AFactor<CanonicalForm> CFAFactor;
+typedef List <CFAFactor> CFAFList;
+typedef ListIterator<CFAFactor> CFAFListIterator;
 typedef Factor<CanonicalForm> CFFactor;
 typedef List<CFFactor> CFFList;
 typedef ListIterator<CFFactor> CFFListIterator;
