@@ -25,9 +25,11 @@
 #include <NTL/ZZX.h>
 #include "NTLconvert.h"
 bool isPurePoly(const CanonicalForm & );
+#ifndef HAVE_FLINT
 static CanonicalForm gcd_univar_ntl0( const CanonicalForm &, const CanonicalForm & );
 static CanonicalForm gcd_univar_ntlp( const CanonicalForm &, const CanonicalForm & );
 #endif
+#endif
 
 #ifdef HAVE_FLINT
 #include "FLINTconvert.h"
@@ -337,30 +339,8 @@ extgcd ( const CanonicalForm & f, const CanonicalForm & g, CanonicalForm & a, Ca
     if (fac_NTL_char!=getCharacteristic())
     {
       fac_NTL_char=getCharacteristic();
-      #ifdef NTL_ZZ
-      ZZ r;
-      r=getCharacteristic();
-      ZZ_pContext ccc(r);
-      #else
-      zz_pContext ccc(getCharacteristic());
-      #endif
-      ccc.restore();
-      #ifdef NTL_ZZ
-      ZZ_p::init(r);
-      #else
       zz_p::init(getCharacteristic());
-      #endif
     }
-    #ifdef NTL_ZZ
-    ZZ_pX F1=convertFacCF2NTLZZpX(f);
-    ZZ_pX G1=convertFacCF2NTLZZpX(g);
-    ZZ_pX R;
-    ZZ_pX A,B;
-    XGCD(R,A,B,F1,G1);
-    a=convertNTLZZpX2CF(A,f.mvar());
-    b=convertNTLZZpX2CF(B,f.mvar());
-    return convertNTLZZpX2CF(R,f.mvar());
-    #else
     zz_pX F1=convertFacCF2NTLzzpX(f);
     zz_pX G1=convertFacCF2NTLzzpX(g);
     zz_pX R;
@@ -369,7 +349,6 @@ extgcd ( const CanonicalForm & f, const CanonicalForm & g, CanonicalForm & a, Ca
     a=convertNTLzzpX2CF(A,f.mvar());
     b=convertNTLzzpX2CF(B,f.mvar());
     return convertNTLzzpX2CF(R,f.mvar());
-    #endif
   }
 #endif
 #ifdef HAVE_FLINT
@@ -410,24 +389,46 @@ extgcd ( const CanonicalForm & f, const CanonicalForm & g, CanonicalForm & a, Ca
     {
       XGCD(RR,A,B,F1,G1,1);
       CanonicalForm rr=convertZZ2CF(RR);
-      ASSERT (!rr.isZero(), "NTL:XGCD failed");
-      a=convertNTLZZX2CF(A,f.mvar())*fc/rr;
-      b=convertNTLZZX2CF(B,f.mvar())*gc/rr;
-      return CanonicalForm(1);
+      if(!rr.isZero())
+      {
+        a=convertNTLZZX2CF(A,f.mvar())*fc/rr;
+        b=convertNTLZZX2CF(B,f.mvar())*gc/rr;
+        return CanonicalForm(1);
+      }
+      else
+      {
+        Off(SW_USE_NTL_GCD_0);
+        r=extgcd(f,g,a,b);
+        if (isOn(SW_RATIONAL))
+        {
+          a/=r.lc();
+          b/=r.lc();
+          r/=r.lc();
+        }
+        On(SW_USE_NTL_GCD_0);
+      }
     }
     else
     {
-      fc=bCommonDen(f);
-      gc=bCommonDen(g);
-      F1=convertFacCF2NTLZZX(f*fc/r);
-      G1=convertFacCF2NTLZZX(g*gc/r);
       XGCD(RR,A,B,F1,G1,1);
-      a=convertNTLZZX2CF(A,f.mvar())*fc;
-      b=convertNTLZZX2CF(B,f.mvar())*gc;
       CanonicalForm rr=convertZZ2CF(RR);
-      ASSERT (!rr.isZero(), "NTL:XGCD failed");
-      a /= rr;
-      b /= rr;
+      if (!rr.isZero())
+      {
+        a=convertNTLZZX2CF(A,f.mvar())*fc;
+        b=convertNTLZZX2CF(B,f.mvar())*gc;
+      }
+      else
+      {
+        Off(SW_USE_NTL_GCD_0);
+        r=extgcd(f,g,a,b);
+        if (isOn(SW_RATIONAL))
+        {
+          a/=r.lc();
+          b/=r.lc();
+          r/=r.lc();
+        }
+        On(SW_USE_NTL_GCD_0);
+      }
       return r;
     }
   }
@@ -475,11 +476,12 @@ extgcd ( const CanonicalForm & f, const CanonicalForm & g, CanonicalForm & a, Ca
 // Used by gcd_poly_univar0().
 //
 //}}}
+
 static CanonicalForm
-balance_p ( const CanonicalForm & f, const CanonicalForm & q )
+balance_p ( const CanonicalForm & f, const CanonicalForm & q, const CanonicalForm & qh )
 {
     Variable x = f.mvar();
-    CanonicalForm result = 0, qh = q / 2;
+    CanonicalForm result = 0;
     CanonicalForm c;
     CFIterator i;
     for ( i = f; i.hasTerms(); i++ )
@@ -493,11 +495,18 @@ balance_p ( const CanonicalForm & f, const CanonicalForm & q )
             result += power( x, i.exp() ) * c;
         }
         else
-          result += power( x, i.exp() ) * balance_p(c,q);
+          result += power( x, i.exp() ) * balance_p(c,q,qh);
     }
     return result;
 }
 
+static CanonicalForm
+balance_p ( const CanonicalForm & f, const CanonicalForm & q )
+{
+    CanonicalForm qh = q / 2;
+    return balance_p (f, q, qh);
+}
+
 /*static CanonicalForm
 balance ( const CanonicalForm & f, const CanonicalForm & q )
 {
@@ -604,9 +613,11 @@ static CanonicalForm gcd_poly_univar0( const CanonicalForm & F, const CanonicalF
 static CanonicalForm
 gcd_poly_p( const CanonicalForm & f, const CanonicalForm & g )
 {
+    if (f.inCoeffDomain() || g.inCoeffDomain()) //zero case should be caught by gcd
+      return 1;
     CanonicalForm pi, pi1;
     CanonicalForm C, Ci, Ci1, Hi, bi, pi2;
-    bool bpure;
+    bool bpure, ezgcdon= isOn (SW_USE_EZGCD_P);
     int delta = degree( f ) - degree( g );
 
     if ( delta >= 0 )
@@ -617,8 +628,28 @@ gcd_poly_p( const CanonicalForm & f, const CanonicalForm & g )
     {
         pi = g; pi1 = f; delta = -delta;
     }
-    Ci = content( pi ); Ci1 = content( pi1 );
-    pi1 = pi1 / Ci1; pi = pi / Ci;
+    if (pi.isUnivariate())
+      Ci= 1;
+    else
+    {
+      if (!ezgcdon)
+        On (SW_USE_EZGCD_P);
+      Ci = content( pi );
+      if (!ezgcdon)
+        Off (SW_USE_EZGCD_P);
+      pi = pi / Ci;
+    }
+    if (pi1.isUnivariate())
+      Ci1= 1;
+    else
+    {
+      if (!ezgcdon)
+        On (SW_USE_EZGCD_P);
+      Ci1 = content( pi1 );
+      if (!ezgcdon)
+        Off (SW_USE_EZGCD_P);
+      pi1 = pi1 / Ci1;
+    }
     C = gcd( Ci, Ci1 );
     int d= 0;
     if ( !( pi.isUnivariate() && pi1.isUnivariate() ) )
@@ -680,7 +711,7 @@ gcd_poly_p( const CanonicalForm & f, const CanonicalForm & g )
         pi2 = pi2 / bi;
         pi = pi1; pi1 = pi2;
         maxNumVars= tmax (getNumVars (pi), getNumVars (pi1));
-        if (!(pi1.isUnivariate()) && (size (pi1)/maxNumVars > 500))
+        if (!pi1.isUnivariate() && (size (pi1)/maxNumVars > 500))
         {
             On (SW_USE_FF_MOD_GCD);
             C *= gcd (oldPi, oldPi1);
@@ -698,7 +729,7 @@ gcd_poly_p( const CanonicalForm & f, const CanonicalForm & g )
             Hi = power( LC( pi1, v ), delta ) / powHi;
             if (!(pi.isUnivariate() && pi1.isUnivariate()))
             {
-              if (size (Hi)*size (pi)/(maxNumVars*3) > 500) //maybe this needs more tuning
+              if (size (Hi)*size (pi)/(maxNumVars*3) > 1500) //maybe this needs more tuning
               {
                 On (SW_USE_FF_MOD_GCD);
                 C *= gcd (oldPi, oldPi1);
@@ -714,7 +745,17 @@ gcd_poly_p( const CanonicalForm & f, const CanonicalForm & g )
       //out_cf("GCD:",C,"\n");
       return C;
     }
-    pi /= content( pi );
+    if (!pi.isUnivariate())
+    {
+      if (!ezgcdon)
+        On (SW_USE_EZGCD_P);
+      Ci= gcd (LC (oldPi,v), LC (oldPi1,v));
+      pi /= LC (pi,v)/Ci;
+      Ci= content (pi);
+      pi /= Ci;
+      if (!ezgcdon)
+        Off (SW_USE_EZGCD_P);
+    }
     if ( bpure )
         pi /= pi.lc();
     C=abs(C*pi);
@@ -1067,7 +1108,7 @@ lcm ( const CanonicalForm & f, const CanonicalForm & g )
 //}}}
 
 #ifdef HAVE_NTL
-
+#ifndef HAVE_FLINT
 static CanonicalForm
 gcd_univar_ntl0( const CanonicalForm & F, const CanonicalForm & G )
 {
@@ -1083,33 +1124,14 @@ gcd_univar_ntlp( const CanonicalForm & F, const CanonicalForm & G )
   if (fac_NTL_char!=getCharacteristic())
   {
     fac_NTL_char=getCharacteristic();
-    #ifdef NTL_ZZ
-    ZZ r;
-    r=getCharacteristic();
-    ZZ_pContext ccc(r);
-    #else
-    zz_pContext ccc(getCharacteristic());
-    #endif
-    ccc.restore();
-    #ifdef NTL_ZZ
-    ZZ_p::init(r);
-    #else
     zz_p::init(getCharacteristic());
-    #endif
   }
-  #ifdef NTL_ZZ
-  ZZ_pX F1=convertFacCF2NTLZZpX(F);
-  ZZ_pX G1=convertFacCF2NTLZZpX(G);
-  ZZ_pX R=GCD(F1,G1);
-  return  convertNTLZZpX2CF(R,F.mvar());
-  #else
   zz_pX F1=convertFacCF2NTLzzpX(F);
   zz_pX G1=convertFacCF2NTLzzpX(G);
   zz_pX R=GCD(F1,G1);
   return  convertNTLzzpX2CF(R,F.mvar());
-  #endif
 }
-
+#endif
 #endif
 
 #ifdef HAVE_FLINT
@@ -1232,11 +1254,10 @@ cf_prepgcd( const CanonicalForm & f, const CanonicalForm & g, int & cc, int & p1
 
 TIMING_DEFINE_PRINT(chinrem_termination)
 TIMING_DEFINE_PRINT(chinrem_recursion)
-TIMING_DEFINE_PRINT(chinrem_reconstruction)
 
 CanonicalForm chinrem_gcd ( const CanonicalForm & FF, const CanonicalForm & GG )
 {
-  CanonicalForm f, g, cl, q(0), Dp, newD, D, newq;
+  CanonicalForm f, g, cl, q(0), Dp, newD, D, newq, newqh;
   int p, i, dp_deg, d_deg=-1;
 
   CanonicalForm cd ( bCommonDen( FF ));
@@ -1256,6 +1277,9 @@ CanonicalForm chinrem_gcd ( const CanonicalForm & FF, const CanonicalForm & GG )
   //g /=vcontent(g,Variable(1));
 
   CanonicalForm Dn, test= 0;
+  CanonicalForm lcf, lcg;
+  lcf= f.lc();
+  lcg= g.lc();
   cl =  gcd (f.lc(),g.lc());
   CanonicalForm gcdcfcg= gcd (cf, cg);
   CanonicalForm fp, gp;
@@ -1285,7 +1309,7 @@ CanonicalForm chinrem_gcd ( const CanonicalForm & FF, const CanonicalForm & GG )
   bool equal= false;
   i = cf_getNumBigPrimes() - 1;
 
-  CanonicalForm cof, cog, cofp, cogp, newCof, newCog, cofn, cogn;
+  CanonicalForm cof, cog, cofp, cogp, newCof, newCog, cofn, cogn, cDn;
   int maxNumVars= tmax (getNumVars (f), getNumVars (g));
   //Off (SW_RATIONAL);
   while ( true )
@@ -1319,8 +1343,11 @@ CanonicalForm chinrem_gcd ( const CanonicalForm & FF, const CanonicalForm & GG )
     TIMING_END_AND_PRINT (chinrem_recursion,
                           "time for gcd mod p in modular gcd: ");
     Dp /=Dp.lc();
+    Dp *= mapinto (cl);
     cofp /= lc (cofp);
+    cofp *= mapinto (lcf);
     cogp /= lc (cogp);
+    cogp *= mapinto (lcg);
     setCharacteristic( 0 );
     dp_deg=totaldegree(Dp);
     if ( dp_deg == 0 )
@@ -1335,6 +1362,9 @@ CanonicalForm chinrem_gcd ( const CanonicalForm & FF, const CanonicalForm & GG )
       cog= mapinto (cogp);
       d_deg=dp_deg;
       q = p;
+      Dn= balance_p (D, p);
+      cofn= balance_p (cof, p);
+      cogn= balance_p (cog, p);
     }
     else
     {
@@ -1345,6 +1375,14 @@ CanonicalForm chinrem_gcd ( const CanonicalForm & FF, const CanonicalForm & GG )
         chineseRemainder( cog, q, mapinto (cogp), p, newCog, newq);
         cof= newCof;
         cog= newCog;
+        newqh= newq/2;
+        Dn= balance_p (newD, newq, newqh);
+        cofn= balance_p (newCof, newq, newqh);
+        cogn= balance_p (newCog, newq, newqh);
+        if (test != Dn) //balance_p (newD, newq))
+          test= balance_p (newD, newq);
+        else
+          equal= true;
         q = newq;
         D = newD;
       }
@@ -1359,6 +1397,9 @@ CanonicalForm chinrem_gcd ( const CanonicalForm & FF, const CanonicalForm & GG )
         d_deg=dp_deg;
         test= 0;
         equal= false;
+        Dn= balance_p (D, p);
+        cofn= balance_p (cof, p);
+        cogn= balance_p (cog, p);
       }
       else
       {
@@ -1368,25 +1409,12 @@ CanonicalForm chinrem_gcd ( const CanonicalForm & FF, const CanonicalForm & GG )
     }
     if ( i >= 0 )
     {
-      TIMING_START (chinrem_reconstruction);
-      Dn= Farey(D,q);
-      cofn= Farey(cof,q);
-      cogn= Farey(cog,q);
-      TIMING_END_AND_PRINT (chinrem_reconstruction,
-                           "time for rational reconstruction in modular gcd: ");
-      int is_rat= isOn (SW_RATIONAL);
-      On (SW_RATIONAL);
-      cd = bCommonDen( Dn ); // we need On(SW_RATIONAL)
-      cofn *= bCommonDen (cofn);
-      cogn *= bCommonDen (cogn);
-      if (!is_rat)
-        Off (SW_RATIONAL);
-      Dn *=cd;
-      if (test != Dn)
-        test= Dn;
-      else
-        equal= true;
-      //Dn /=vcontent(Dn,Variable(1));
+      cDn= icontent (Dn);
+      Dn /= cDn;
+      cofn /= cl/cDn;
+      //cofn /= icontent (cofn);
+      cogn /= cl/cDn;
+      //cogn /= icontent (cogn);
       TIMING_START (chinrem_termination);
       if ((terminationTest (f,g, cofn, cogn, Dn)) ||
           ((equal || q > b) && fdivides (Dn, f) && fdivides (Dn, g)))
