@@ -49,7 +49,7 @@ ratBiSqrfFactorize (const CanonicalForm & G,        ///< [in] a bivariate poly
 {
   CFMap N;
   CanonicalForm F= compress (G, N);
-  CanonicalForm contentX= content (F, 1);
+  CanonicalForm contentX= content (F, 1); //erwarte hier primitiven input: primitiv über Z bzw. Z[a]
   CanonicalForm contentY= content (F, 2);
   F /= (contentX*contentY);
   CFFList contentXFactors, contentYFactors;
@@ -81,8 +81,17 @@ ratBiSqrfFactorize (const CanonicalForm & G,        ///< [in] a bivariate poly
     }
     return result;
   }
-  mat_ZZ M;
-  vec_ZZ S;
+
+  mpz_t * M=new mpz_t [4];
+  mpz_init (M[0]);
+  mpz_init (M[1]);
+  mpz_init (M[2]);
+  mpz_init (M[3]);
+
+  mpz_t * S=new mpz_t [2];
+  mpz_init (S[0]);
+  mpz_init (S[1]);
+
   F= compress (F, M, S);
   CFList result= biFactorize (F, v);
   for (CFListIterator i= result; i.hasItem(); i++)
@@ -96,6 +105,17 @@ ratBiSqrfFactorize (const CanonicalForm & G,        ///< [in] a bivariate poly
     normalize (result);
     result.insert (Lc (G));
   }
+
+  mpz_clear (M[0]);
+  mpz_clear (M[1]);
+  mpz_clear (M[2]);
+  mpz_clear (M[3]);
+  delete [] M;
+
+  mpz_clear (S[0]);
+  mpz_clear (S[1]);
+  delete [] S;
+
   return result;
 }
 
@@ -195,8 +215,17 @@ ratBiFactorize (const CanonicalForm & G,         ///< [in] a bivariate poly
     }
     return result;
   }
-  mat_ZZ M;
-  vec_ZZ S;
+
+  mpz_t * M=new mpz_t [4];
+  mpz_init (M[0]);
+  mpz_init (M[1]);
+  mpz_init (M[2]);
+  mpz_init (M[3]);
+
+  mpz_t * S=new mpz_t [2];
+  mpz_init (S[0]);
+  mpz_init (S[1]);
+
   F= compress (F, M, S);
   TIMING_START (fac_bi_sqrf);
   CFFList sqrfFactors= sqrFree (F);
@@ -231,6 +260,17 @@ ratBiFactorize (const CanonicalForm & G,         ///< [in] a bivariate poly
     }
     result.insert (CFFactor (LcF, 1));
   }
+
+  mpz_clear (M[0]);
+  mpz_clear (M[1]);
+  mpz_clear (M[2]);
+  mpz_clear (M[3]);
+  delete [] M;
+
+  mpz_clear (S[0]);
+  mpz_clear (S[1]);
+  delete [] S;
+
   return result;
 }
 
