@@ -6,10 +6,8 @@
  * This file implements functions to lift factors via Hensel lifting.
  *
  * ABSTRACT: Hensel lifting is described in "Efficient Multivariate
- * Factorization over Finite Fields" by L. Bernardin & M. Monagon. Division with
- * remainder is described in "Fast Recursive Division" by C. Burnikel and
- * J. Ziegler. Karatsuba multiplication is described in "Modern Computer
- * Algebra" by J. von zur Gathen and J. Gerhard.
+ * Factorization over Finite Fields" by L. Bernardin & M. Monagon and 
+ * "Algorithms for Computer Algebra" by Geddes, Czapor, Labahn
  *
  * @author Martin Lee
  *
@@ -33,6 +31,12 @@
 #include <NTL/lzz_pEX.h>
 #include "NTLconvert.h"
 
+TIMING_DEFINE_PRINT (diotime)
+TIMING_DEFINE_PRINT (product1)
+TIMING_DEFINE_PRINT (product2)
+TIMING_DEFINE_PRINT (hensel23)
+TIMING_DEFINE_PRINT (hensel)
+
 static
 CFList productsNTL (const CFList& factors, const CanonicalForm& M)
 {
@@ -481,6 +485,8 @@ diophantineHensel (const CanonicalForm & F, const CFList& factors,
   return result;
 }
 
+/// solve \f$ 1=\sum_{i=1}^n{\delta_{i} \prod_{j\neq i}{f_j}} \f$ mod \f$p^k\f$
+/// over Q(alpha) by p-adic lifting
 CFList
 diophantineHenselQa (const CanonicalForm & F, const CanonicalForm& G,
                      const CFList& factors, modpk& b, const Variable& alpha)
@@ -689,6 +695,10 @@ diophantineHenselQa (const CanonicalForm & F, const CanonicalForm& G,
   return result;
 }
 
+
+/// solve \f$ 1=\sum_{i=1}^n{\delta_{i} \prod_{j\neq i}{f_j}} \f$ mod \f$p^k\f$
+/// over Q(alpha) by first computing mod \f$p\f$ and if no zero divisor occured
+/// compute it mod \f$p^k\f$
 CFList
 diophantineQa (const CanonicalForm& F, const CanonicalForm& G,
                const CFList& factors, modpk& b, const Variable& alpha)
@@ -889,7 +899,7 @@ henselStep12 (const CanonicalForm& F, const CFList& factors,
     if (degree (bufFactors[k], x) > 0)
     {
       if (k > 0)
-        remainder= modNTL (E, bufFactors[k] [0], b);
+        remainder= modNTL (E, bufFactors[k] [0], b); //TODO precompute inverses of bufFactors[k][0]
       else
         remainder= E;
     }
@@ -915,40 +925,22 @@ henselStep12 (const CanonicalForm& F, const CFList& factors,
   if (degBuf0 > 0 && degBuf1 > 0)
     M (j + 1, 1)= mulNTL (bufFactors[0] [j], bufFactors[1] [j], b);
   CanonicalForm uIZeroJ;
-  if (j == 1)
-  {
-    if (degBuf0 > 0 && degBuf1 > 0)
-      uIZeroJ= mulNTL ((bufFactors[0] [0] + bufFactors[0] [j]),
-                  (bufFactors[1] [0] + buf[1]), b) - M(1, 1) - M(j + 1, 1);
-    else if (degBuf0 > 0)
-      uIZeroJ= mulNTL (bufFactors[0] [j], bufFactors[1], b);
-    else if (degBuf1 > 0)
-      uIZeroJ= mulNTL (bufFactors[0], buf[1], b);
-    else
-      uIZeroJ= 0;
-    if (b.getp() != 0)
-      uIZeroJ= b (uIZeroJ);
-    Pi [0] += xToJ*uIZeroJ;
-    if (b.getp() != 0)
-      Pi [0]= b (Pi[0]);
-  }
+
+  if (degBuf0 > 0 && degBuf1 > 0)
+    uIZeroJ= mulNTL ((bufFactors[0] [0] + bufFactors[0] [j]),
+                    (bufFactors[1] [0] + buf[1]), b) - M(1, 1) - M(j + 1, 1);
+  else if (degBuf0 > 0)
+    uIZeroJ= mulNTL (bufFactors[0] [j], bufFactors[1], b);
+  else if (degBuf1 > 0)
+    uIZeroJ= mulNTL (bufFactors[0], buf[1], b);
   else
-  {
-    if (degBuf0 > 0 && degBuf1 > 0)
-      uIZeroJ= mulNTL ((bufFactors[0] [0] + bufFactors[0] [j]),
-                  (bufFactors[1] [0] + buf[1]), b) - M(1, 1) - M(j + 1, 1);
-    else if (degBuf0 > 0)
-      uIZeroJ= mulNTL (bufFactors[0] [j], bufFactors[1], b);
-    else if (degBuf1 > 0)
-      uIZeroJ= mulNTL (bufFactors[0], buf[1], b);
-    else
-      uIZeroJ= 0;
-    if (b.getp() != 0)
-      uIZeroJ= b (uIZeroJ);
-    Pi [0] += xToJ*uIZeroJ;
-    if (b.getp() != 0)
-      Pi [0]= b (Pi[0]);
-  }
+    uIZeroJ= 0;
+  if (b.getp() != 0)
+    uIZeroJ= b (uIZeroJ);
+  Pi [0] += xToJ*uIZeroJ;
+  if (b.getp() != 0)
+    Pi [0]= b (Pi[0]);
+
   CFArray tmp= CFArray (factors.length() - 1);
   for (k= 0; k < factors.length() - 1; k++)
     tmp[k]= 0;
@@ -1442,32 +1434,17 @@ henselStep (const CanonicalForm& F, const CFList& factors, CFArray& bufFactors,
   if (degBuf0 > 0 && degBuf1 > 0)
     M (j + 1, 1)= mulMod (bufFactors[0] [j], bufFactors[1] [j], MOD);
   CanonicalForm uIZeroJ;
-  if (j == 1)
-  {
-    if (degBuf0 > 0 && degBuf1 > 0)
-      uIZeroJ= mulMod ((bufFactors[0] [0] + bufFactors[0] [j]),
+
+  if (degBuf0 > 0 && degBuf1 > 0)
+    uIZeroJ= mulMod ((bufFactors[0] [0] + bufFactors[0] [j]),
                   (bufFactors[1] [0] + buf[1]), MOD) - M(1, 1) - M(j + 1, 1);
-    else if (degBuf0 > 0)
-      uIZeroJ= mulMod (bufFactors[0] [j], bufFactors[1], MOD);
-    else if (degBuf1 > 0)
-      uIZeroJ= mulMod (bufFactors[0], buf[1], MOD);
-    else
-      uIZeroJ= 0;
-    Pi [0] += xToJ*uIZeroJ;
-  }
+  else if (degBuf0 > 0)
+    uIZeroJ= mulMod (bufFactors[0] [j], bufFactors[1], MOD);
+  else if (degBuf1 > 0)
+    uIZeroJ= mulMod (bufFactors[0], buf[1], MOD);
   else
-  {
-    if (degBuf0 > 0 && degBuf1 > 0)
-      uIZeroJ= mulMod ((bufFactors[0] [0] + bufFactors[0] [j]),
-                  (bufFactors[1] [0] + buf[1]), MOD) - M(1, 1) - M(j + 1, 1);
-    else if (degBuf0 > 0)
-      uIZeroJ= mulMod (bufFactors[0] [j], bufFactors[1], MOD);
-    else if (degBuf1 > 0)
-      uIZeroJ= mulMod (bufFactors[0], buf[1], MOD);
-    else
-      uIZeroJ= 0;
-    Pi [0] += xToJ*uIZeroJ;
-  }
+    uIZeroJ= 0;
+  Pi [0] += xToJ*uIZeroJ;
 
   CFArray tmp= CFArray (factors.length() - 1);
   for (k= 0; k < factors.length() - 1; k++)
@@ -1788,15 +1765,23 @@ nonMonicHenselStep12 (const CanonicalForm& F, const CFList& factors,
     if (j + 2 <= M.rows())
       M (j + 2, 1)= mulNTL (bufFactors[0] [j + 1], bufFactors[1] [j + 1]);
   }
+  else
+    M (j + 1, 1)= 0;
+
   CanonicalForm uIZeroJ;
   if (degBuf0 > 0 && degBuf1 > 0)
-    uIZeroJ= mulNTL(bufFactors[0][0],buf[1])+mulNTL (bufFactors[1][0], buf[0]);
+    uIZeroJ= mulNTL(bufFactors[0][0], buf[1]) +
+             mulNTL (bufFactors[1][0], buf[0]);
   else if (degBuf0 > 0)
-    uIZeroJ= mulNTL (buf[0], bufFactors[1]);
+    uIZeroJ= mulNTL (buf[0], bufFactors[1]) +
+             mulNTL (buf[1], bufFactors[0][0]);
   else if (degBuf1 > 0)
-    uIZeroJ= mulNTL (bufFactors[0], buf [1]);
+    uIZeroJ= mulNTL (bufFactors[0], buf[1]) +
+             mulNTL (buf[0], bufFactors[1][0]);
   else
-    uIZeroJ= 0;
+    uIZeroJ= mulNTL (bufFactors[0], buf[1]) +
+             mulNTL (buf[0], bufFactors[1]);
+
   Pi [0] += xToJ*uIZeroJ;
 
   CFArray tmp= CFArray (factors.length() - 1);
@@ -1811,22 +1796,20 @@ nonMonicHenselStep12 (const CanonicalForm& F, const CFList& factors,
     while (two.hasTerms() && two.exp() > j) two++;
     for (k= 1; k <= (int) ceil (j/2.0); k++)
     {
-      if (one.hasTerms() && two.hasTerms())
+      if (k != j - k + 1)
       {
-        if (k != j - k + 1)
-        {
-          if ((one.hasTerms() && one.exp() == j - k + 1) && +
-              (two.hasTerms() && two.exp() == j - k + 1))
+        if ((one.hasTerms() && one.exp() == j - k + 1) && +
+            (two.hasTerms() && two.exp() == j - k + 1))
         {
           tmp[0] += mulNTL ((bufFactors[0][k]+one.coeff()),(bufFactors[1][k] +
-                      two.coeff())) - M (k + 1, 1) - M (j - k + 2, 1);
+                    two.coeff())) - M (k + 1, 1) - M (j - k + 2, 1);
           one++;
           two++;
         }
         else if (one.hasTerms() && one.exp() == j - k + 1)
         {
           tmp[0] += mulNTL ((bufFactors[0][k]+one.coeff()), bufFactors[1] [k]) -
-                      M (k + 1, 1);
+                    M (k + 1, 1);
           one++;
         }
         else if (two.hasTerms() && two.exp() == j - k + 1)
@@ -1838,7 +1821,6 @@ nonMonicHenselStep12 (const CanonicalForm& F, const CFList& factors,
       }
       else
         tmp[0] += M (k + 1, 1);
-      }
     }
   }
 
@@ -1877,16 +1859,21 @@ nonMonicHenselStep12 (const CanonicalForm& F, const CFList& factors,
       if (j + 2 <= M.rows())
         M (j + 2, l + 1)= mulNTL (Pi [l - 1][j + 1], bufFactors[l + 1] [j + 1]);
     }
+    else
+      M (j + 1, l + 1)= 0;
 
     if (degPi > 0 && degBuf > 0)
-      uIZeroJ= mulNTL (Pi[l -1] [0], buf[l + 1]) +
+      uIZeroJ= mulNTL (Pi[l - 1] [0], buf[l + 1]) +
                mulNTL (uIZeroJ, bufFactors[l+1] [0]);
     else if (degPi > 0)
-      uIZeroJ= mulNTL (uIZeroJ, bufFactors[l + 1]);
+      uIZeroJ= mulNTL (uIZeroJ, bufFactors[l + 1]) +
+               mulNTL (Pi[l - 1][0], buf[l + 1]);
     else if (degBuf > 0)
-      uIZeroJ= mulNTL (Pi[l - 1], buf[1]);
+      uIZeroJ= mulNTL (uIZeroJ, bufFactors[l + 1][0]) +
+               mulNTL (Pi[l - 1], buf[l + 1]);
     else
-      uIZeroJ= 0;
+      uIZeroJ= mulNTL (uIZeroJ, bufFactors[l + 1]) +
+               mulNTL (Pi[l - 1], buf[l + 1]);
 
     Pi [l] += xToJ*uIZeroJ;
 
@@ -2032,8 +2019,8 @@ nonMonicHenselLift12 (const CanonicalForm& F, CFList& factors, int l,
 }
 
 
-/// solve \f$ E=sum_{i= 1}^{r}{\sigma_{i}prod_{j=1, j\neq i}^{r}{f_{i}}}\f$
-/// mod M, products contains \f$ prod_{j=1, j\neq i}^{r}{f_{i}}} \f$
+/// solve \f$ E=\sum_{i= 1}^r{\sigma_{i}\prod_{j=1, j\neq i}^{r}{f_{i}}}\f$
+/// mod M, @a products contains \f$ \prod_{j=1, j\neq i}^{r}{f_{j}} \f$
 CFList
 diophantine (const CFList& recResult, const CFList& factors,
              const CFList& products, const CFList& M, const CanonicalForm& E,
@@ -2076,7 +2063,7 @@ diophantine (const CFList& recResult, const CFList& factors,
   CanonicalForm e= E;
   CFListIterator j= products;
   for (CFListIterator i= recDiophantine; i.hasItem(); i++, j++)
-    e -= i.getItem()*j.getItem();
+    e -= j.getItem()*i.getItem();
 
   CFList result= recDiophantine;
   int d= degree (M.getLast());
@@ -2098,7 +2085,7 @@ diophantine (const CFList& recResult, const CFList& factors,
       for (j= recDiophantine; j.hasItem(); j++, k++, l++)
       {
         k.getItem() += j.getItem()*power (y, i);
-        e -= j.getItem()*power (y, i)*l.getItem();
+        e -= l.getItem()*(j.getItem()*power (y, i));
       }
     }
     if (e.isZero())
@@ -2146,6 +2133,7 @@ nonMonicHenselStep (const CanonicalForm& F, const CFList& factors,
   CFArray buf= CFArray (diophant.length());
 
   // actual lifting
+  TIMING_START (diotime);
   CFList diophantine2= diophantine (diophant, factors, products, MOD, E,
                                     noOneToOne);
 
@@ -2158,8 +2146,10 @@ nonMonicHenselStep (const CanonicalForm& F, const CFList& factors,
     buf[k]= i.getItem();
     bufFactors[k] += xToJ*i.getItem();
   }
+  TIMING_END_AND_PRINT (diotime, "time for dio: ");
 
   // update Pi [0]
+  TIMING_START (product2);
   int degBuf0= degree (bufFactors[0], x);
   int degBuf1= degree (bufFactors[1], x);
   if (degBuf0 > 0 && degBuf1 > 0)
@@ -2168,17 +2158,22 @@ nonMonicHenselStep (const CanonicalForm& F, const CFList& factors,
     if (j + 2 <= M.rows())
       M (j + 2, 1)= mulMod (bufFactors[0] [j + 1], bufFactors[1] [j + 1], MOD);
   }
-  CanonicalForm uIZeroJ;
+  else
+    M (j + 1, 1)= 0;
 
+  CanonicalForm uIZeroJ;
   if (degBuf0 > 0 && degBuf1 > 0)
     uIZeroJ= mulMod (bufFactors[0] [0], buf[1], MOD) +
              mulMod (bufFactors[1] [0], buf[0], MOD);
   else if (degBuf0 > 0)
-    uIZeroJ= mulMod (buf[0], bufFactors[1], MOD);
+    uIZeroJ= mulMod (buf[0], bufFactors[1], MOD) +
+             mulMod (buf[1], bufFactors[0][0], MOD);
   else if (degBuf1 > 0)
-    uIZeroJ= mulMod (bufFactors[0], buf[1], MOD);
+    uIZeroJ= mulMod (bufFactors[0], buf[1], MOD) +
+             mulMod (buf[0], bufFactors[1][0], MOD);
   else
-    uIZeroJ= 0;
+    uIZeroJ= mulMod (bufFactors[0], buf[1], MOD) +
+             mulMod (buf[0], bufFactors[1], MOD);
   Pi [0] += xToJ*uIZeroJ;
 
   CFArray tmp= CFArray (factors.length() - 1);
@@ -2260,16 +2255,21 @@ nonMonicHenselStep (const CanonicalForm& F, const CFList& factors,
         M (j + 2, l + 1)= mulMod (Pi [l - 1] [j + 1], bufFactors[l + 1] [j + 1],
                                   MOD);
     }
+    else
+      M (j + 1, l + 1)= 0;
 
     if (degPi > 0 && degBuf > 0)
-      uIZeroJ= mulMod (Pi[l -1] [0], buf[l + 1], MOD) +
-               mulMod (uIZeroJ, bufFactors[l+1] [0], MOD);
+      uIZeroJ= mulMod (Pi[l - 1] [0], buf[l + 1], MOD) +
+               mulMod (uIZeroJ, bufFactors[l + 1] [0], MOD);
     else if (degPi > 0)
-      uIZeroJ= mulMod (uIZeroJ, bufFactors[l + 1], MOD);
+      uIZeroJ= mulMod (uIZeroJ, bufFactors[l + 1], MOD) +
+               mulMod (Pi[l - 1][0], buf[l + 1], MOD);
     else if (degBuf > 0)
-      uIZeroJ= mulMod (Pi[l - 1], buf[1], MOD);
+      uIZeroJ= mulMod (Pi[l - 1], buf[l + 1], MOD) +
+               mulMod (uIZeroJ, bufFactors[l + 1][0], MOD);
     else
-      uIZeroJ= 0;
+      uIZeroJ= mulMod (Pi[l - 1], buf[l + 1], MOD) +
+               mulMod (uIZeroJ, bufFactors[l + 1], MOD);
 
     Pi [l] += xToJ*uIZeroJ;
 
@@ -2334,6 +2334,7 @@ nonMonicHenselStep (const CanonicalForm& F, const CFList& factors,
 
     Pi[l] += tmp[l]*xToJ*F.mvar();
   }
+  TIMING_END_AND_PRINT (product2, "time for product in hensel step: ");
   return;
 }
 
@@ -2544,7 +2545,9 @@ nonMonicHenselLift23 (const CanonicalForm& F, const CFList& factors, const
   bufF= mod (bufF, y);
   bufF= mod (bufF, Variable (3));
 
+  TIMING_START (diotime);
   diophant= diophantine (bufF, bufFactors2);
+  TIMING_END_AND_PRINT (diotime, "time for dio in 23: ");
 
   CFMatrix M= CFMatrix (liftBound, bufFactors2.length() - 1);
 
@@ -2608,8 +2611,10 @@ nonMonicHenselLift23 (const CanonicalForm& F, const CFList& factors, const
 
   CFList products;
   bufF= mod (F, Variable (3));
+  TIMING_START (product1);
   for (CFListIterator k= factors; k.hasItem(); k++)
     products.append (bufF/k.getItem());
+  TIMING_END_AND_PRINT (product1, "time for product1 in 23: ");
 
   CFList MOD= CFList (power (v, liftBound));
   MOD.insert (yToL);
@@ -2672,6 +2677,7 @@ nonMonicHenselLift (const CFList& F, const CFList& factors, const CFList& LCs,
   CFList products;
   CanonicalForm quot, bufF= F.getFirst();
 
+  TIMING_START (product1);
   for (int i= 0; i < bufFactors.size(); i++)
   {
     if (degree (bufFactors[i], y) > 0)
@@ -2693,6 +2699,7 @@ nonMonicHenselLift (const CFList& F, const CFList& factors, const CFList& LCs,
       products.append (quot);
     }
   }
+  TIMING_END_AND_PRINT (product1, "time for product1 in further hensel:" );
 
   for (int d= 1; d < lNew; d++)
   {
@@ -2720,9 +2727,11 @@ nonMonicHenselLift (const CFList& eval, const CFList& factors,
   CFMatrix M= CFMatrix (liftBound[1], factors.length() - 1);
   int k= 0;
 
+  TIMING_START (hensel23);
   CFList result=
   nonMonicHenselLift23 (eval.getFirst(), factors, LCs [0], diophant, bufPi,
                         liftBound[1], liftBound[0], noOneToOne);
+  TIMING_END_AND_PRINT (hensel23, "time for 23: ");
 
   if (noOneToOne)
     return CFList();
@@ -2744,8 +2753,10 @@ nonMonicHenselLift (const CFList& eval, const CFList& factors,
   {
     bufEval.append (j.getItem());
     M= CFMatrix (liftBound[i], factors.length() - 1);
+    TIMING_START (hensel);
     result= nonMonicHenselLift (bufEval, result, LCs [i-1], diophant, bufPi, M,
                                 liftBound[i-1], liftBound[i], MOD, noOneToOne);
+    TIMING_END_AND_PRINT (hensel, "time for further hensel: ");
     if (noOneToOne)
       return result;
     MOD.append (power (Variable (i + 2), liftBound[i]));
