@@ -295,8 +295,18 @@ int Hensel (const CanonicalForm & UU, CFArray & G, const Evaluation & AA,
   LCs [2]= MM (LeadCoeffs [2]);
 
   CFList evaluation;
+  long termEstimate= size (U);
   for (int i= A.min(); i <= A.max(); i++)
+  {
+    if (!A[i].isZero())
+    {
+      termEstimate *= degree (U,i)*2;
+      termEstimate /= 3;
+    }
     evaluation.append (A [i]);
+  }
+  if (termEstimate/getNumVars(U) > 500)
+    return -1;
   CFList UEval;
   CanonicalForm shiftedU= myShift2Zero (U, UEval, evaluation);
 
@@ -419,12 +429,33 @@ ezgcd ( const CanonicalForm & FF, const CanonicalForm & GG, REvaluation & b,
         bool internal )
 {
   bool isRat= isOn (SW_RATIONAL);
+
+  int maxNumVars= tmax (getNumVars (FF), getNumVars (GG));
+  int sizeF= size (FF);
+  int sizeG= size (GG);
+
+
+  if (!isRat)
+    On (SW_RATIONAL);
+  if (sizeF/maxNumVars > 500 && sizeG/maxNumVars > 500)
+  {
+    Off(SW_USE_EZGCD);
+    CanonicalForm result=gcd( FF, GG );
+    On(SW_USE_EZGCD);
+    if (!isRat)
+      Off (SW_RATIONAL);
+    result /= icontent (result);
+    DEBDECLEVEL( cerr, "ezgcd" );
+    return result;
+  }
+
+
   CanonicalForm F, G, f, g, d, Fb, Gb, Db, Fbt, Gbt, Dbt, B0, B, D0, lcF, lcG,
                 lcD, cand, contcand, result;
   CFArray DD( 1, 2 ), lcDD( 1, 2 );
   int degF, degG, delta, t, count, maxeval;
   REvaluation bt;
-  bool gcdfound = false;
+  int gcdfound = 0;
   Variable x = Variable(1);
   count= 0;
   maxeval= 200;
@@ -469,13 +500,24 @@ ezgcd ( const CanonicalForm & FF, const CanonicalForm & GG, REvaluation & b,
     DEBDECLEVEL( cerr, "ezgcd" );
     if(F.mvar()==G.mvar())
       d*=gcd(F,G);
+    else
+      return N (d);
     return N (d);
   }
+  if ( F.isUnivariate())
+  {
+    g= content (G,G.mvar());
+    return N(d*gcd(F,g));
+  }
+  if ( G.isUnivariate())
+  {
+    f= content (F,F.mvar());
+    return N(d*gcd(G,f));
+  }
 
-  int maxNumVars= tmax (getNumVars (F), getNumVars (G));
-  int sizeF= size (F);
-  int sizeG= size (G);
-
+  maxNumVars= tmax (getNumVars (F), getNumVars (G));
+  sizeF= size (F);
+  sizeG= size (G);
 
   if (!isRat)
     On (SW_RATIONAL);
