@@ -19,6 +19,7 @@
 #include <kernel/ffields.h>
 #include <factory/factory.h>
 #include <kernel/clapconv.h>
+#include <kernel/bigintmat.h>
 #include <libfac/factor.h>
 #include <kernel/ring.h>
 
@@ -1092,6 +1093,81 @@ notImpl:
   //PrintS("......S\n");
   return res;
 }
+#ifdef HAVE_NTL
+ideal singclap_absFactorize ( poly f, ideal & mipos, intvec ** exps, int & numFactors)
+{
+  pTest(f);
+
+  ideal res=NULL;
+
+  int offs = rPar(currRing);
+  if (f==NULL)
+  {
+    res= idInit (1, 1);
+    mipos= idInit (1, 1);
+    mipos->m[0]= convFactoryPSingTrP (Variable (offs)); //overkill
+    (*exps)=new intvec (1);
+    (**exps)[0]= 1;
+    numFactors= 0;
+    return res;
+  }
+  CanonicalForm F( convSingTrPFactoryP( f ) );
+
+  bool isRat= isOn (SW_RATIONAL);
+  if (!isRat)
+    On (SW_RATIONAL);
+
+  CFAFList absFactors= absFactorize (F);
+
+  int n= absFactors.length();
+  *exps=new intvec (n);
+
+  res= idInit (n, 1);
+
+  mipos= idInit (n, 1);
+
+  Variable x= Variable (offs);
+  Variable alpha;
+  int i= 0;
+  numFactors= 0;
+  int count;
+  CFAFListIterator iter= absFactors;
+  CanonicalForm lead= iter.getItem().factor();
+  if (iter.getItem().factor().inCoeffDomain())
+  {
+    i++;
+    iter++;
+  }
+  for (; iter.hasItem(); iter++, i++)
+  {
+    (**exps)[i]= iter.getItem().exp();
+    alpha= iter.getItem().minpoly().mvar();
+    if (iter.getItem().minpoly().isOne())
+      lead /= power (bCommonDen (iter.getItem().factor()), iter.getItem().exp());
+    else
+      lead /= power (power (bCommonDen (iter.getItem().factor()), degree (iter.getItem().minpoly())), iter.getItem().exp());
+    res->m[i]= convFactoryPSingTrP (replacevar (iter.getItem().factor()*bCommonDen (iter.getItem().factor()), alpha, x));
+    if (iter.getItem().minpoly().isOne())
+    {
+      count= iter.getItem().exp();
+      mipos->m[i]= convFactoryPSingTrP (x);
+    }
+    else
+    {
+      count= iter.getItem().exp()*degree (iter.getItem().minpoly());
+      mipos->m[i]= convFactoryPSingTrP (replacevar (iter.getItem().minpoly(), alpha, x));
+    }
+    numFactors += count;
+  }
+  if (!isRat)
+    Off (SW_RATIONAL);
+
+  (**exps)[0]= 1;
+  res->m[0]= convFactoryPSingTrP (lead);
+  mipos->m[0]= convFactoryPSingTrP (x);
+  return res;
+}
+#endif
 ideal singclap_sqrfree ( poly f, intvec ** v , int with_exps)
 {
   pTest(f);
@@ -1494,7 +1570,7 @@ char* singclap_neworder ( ideal I)
       if (done) StringAppendS(",");
     }
   }
-  char * s=omStrDup(StringAppendS(""));
+  char * s=StringEndS();
   if (s[strlen(s)-1]==',') s[strlen(s)-1]='\0';
   return s;
 }
@@ -1826,6 +1902,20 @@ void singclap_algdividecontent ( napoly f, napoly g, napoly &ff, napoly &gg )
  Off(SW_RATIONAL);
 }
 
+number singclap_det_bi( bigintmat * m)
+{
+  CFMatrix M(m->rows(),m->cols());
+  int i,j;
+  for(i=m->rows();i>0;i--)
+  {
+    for(j=m->cols();j>0;j--)
+    {
+      M(i,j)=convSingNFactoryN(BIMATELEM(*m,i,j),NULL);
+    }
+  }
+  number res= convFactoryNSingN( determinant(M,m->rows()),NULL ) ;
+  return res;
+}
 #if 0
 lists singclap_chineseRemainder(lists x, lists q)
 {
