@@ -322,12 +322,12 @@ static int fe_getchar()
   {
     /* check for CSI */
     c='\0';
-    read (STDIN_FILENO, &c, 1);
+    while((-1 == read (STDIN_FILENO, &c, 1)) && (errno == EINTR));
     if (c == '[')
     {
       /* get command character */
       c='\0';
-      read (STDIN_FILENO, &c, 1);
+      while((-1 == read (STDIN_FILENO, &c, 1)) && (errno == EINTR));
       switch (c)
       {
         case 'D': /* left arrow key */
@@ -412,11 +412,16 @@ char * fe_fgets_stdin_fe(char *pr,char *s, int size)
           FD_SET(STDIN_FILENO, &fdset);
           tv.tv_sec = 0;
           tv.tv_usec = 0;
-          #ifdef hpux
-            sel = select (STDIN_FILENO+1, (int *)fdset.fds_bits, NULL, NULL, &tv);
-          #else
-            sel = select (STDIN_FILENO+1, &fdset, NULL, NULL, &tv);
-          #endif
+          do
+          {
+            sel = select (STDIN_FILENO+1, 
+#ifdef hpux
+                          (int *)fdset.fds_bits,
+#else
+                          &fdset,
+#endif
+                          NULL, NULL, &tv);
+          } while( (sel == -1) && (errno == EINTR) );
           if (sel==0)
             fe_temp_reset();
           return s;
