@@ -30,6 +30,7 @@
 #include <kernel/ideals.h>
 #include <kernel/prCopy.h>
 #include <kernel/gring.h>
+#include <kernel/polys.h>
 
 
 omBin sip_sideal_bin = omGetSpecBin(sizeof(sip_sideal));
@@ -95,7 +96,7 @@ int idPosConstant(ideal id)
   int k;
   for (k = IDELEMS(id)-1; k>=0; k--)
   {
-    if (p_LmIsConstantComp(id->m[k], currRing) == TRUE)
+    if ((id->m[k]!=NULL) && p_LmIsConstantComp(id->m[k], currRing))
       return k;
   }
   return -1;
@@ -203,6 +204,22 @@ void idSkipZeroes (ideal ide)
   }
 }
 
+/*2
+* keeps the first k (>= 1) entries of the given ideal
+* (Note that the kept polynomials may be zero.)
+*/
+void idKeepFirstK(ideal id, const int k)
+{
+  for (int i = IDELEMS(id)-1; i >= k; i--)
+  {
+    if (id->m[i] != NULL) pDelete(&id->m[i]);
+  }
+  int kk=k;
+  if (k==0) kk=1; /* ideals must have at least one element(0)*/
+  pEnlargeSet(&(id->m), IDELEMS(id), kk-IDELEMS(id));
+  IDELEMS(id) = kk;
+}
+
 /*2
 * copies the first k (>= 1) entries of the given ideal
 * and returns these as a new ideal
@@ -270,30 +287,6 @@ void idDelMultiples(ideal id)
   }
 }
 
-/*2
-* ideal id = (id[i])
-* if id[i] = id[j] then id[j] is deleted for j > i
-*/
-void idDelEquals(ideal id)
-{
-  int i, j;
-  int k = IDELEMS(id)-1;
-  for (i=k; i>=0; i--)
-  {
-    if (id->m[i]!=NULL)
-    {
-      for (j=k; j>i; j--)
-      {
-        if ((id->m[j]!=NULL)
-        && (pEqualPolys(id->m[i], id->m[j])))
-        {
-          pDelete(&id->m[j]);
-        }
-      }
-    }
-  }
-}
-
 //
 // Delete id[j], if Lm(j) == Lm(i) and both LC(j), LC(i) are units and j > i
 //
@@ -470,6 +463,100 @@ static int pComp_RevLex(poly a, poly b,BOOLEAN nolex)
   return -1;
 }
 
+/*
+* compare the leading terms of a and b
+*/
+static int tCompare(const poly a, const poly b)
+{
+  if (b == NULL) return(a != NULL);
+  if (a == NULL) return(-1);
+
+  /* a != NULL && b != NULL */
+  int r = pLmCmp(a, b);
+  if (r != 0) return(r);
+  number h = nSub(pGetCoeff(a), pGetCoeff(b));
+  r = -1 + nIsZero(h) + 2*nGreaterZero(h);   /* -1: <, 0:==, 1: > */
+  nDelete(&h);
+  return(r);
+}
+
+/*
+* compare a and b (rev-lex on terms)
+*/
+static int pCompare(const poly a, const poly b)
+{
+  int r = tCompare(a, b);
+  if (r != 0) return(r);
+
+  poly aa = a;
+  poly bb = b;
+  while (r == 0 && aa != NULL && bb != NULL)
+  {
+    pIter(aa);
+    pIter(bb);
+    r = tCompare(aa, bb);
+  }
+  return(r);
+}
+
+typedef struct
+{
+  poly p;
+  int index;
+} poly_sort;
+
+int pCompare_qsort(const void *a, const void *b)
+{
+  int res = pCompare(((poly_sort *)a)->p, ((poly_sort *)b)->p);
+  return(res);
+}
+
+void idSort_qsort(poly_sort *id_sort, int idsize)
+{
+  qsort(id_sort, idsize, sizeof(poly_sort), pCompare_qsort);
+}
+
+/*2
+* ideal id = (id[i])
+* if id[i] = id[j] then id[j] is deleted for j > i
+*/
+void idDelEquals(ideal id)
+{
+  int idsize = IDELEMS(id);
+  poly_sort *id_sort = (poly_sort *)omAlloc0(idsize*sizeof(poly_sort));
+  for (int i = 0; i < idsize; i++)
+  {
+    id_sort[i].p = id->m[i];
+    id_sort[i].index = i;
+  }
+  idSort_qsort(id_sort, idsize);
+  int index, index_i, index_j;
+  int i = 0;
+  for (int j = 1; j < idsize; j++)
+  {
+    if (id_sort[i].p != NULL && pEqualPolys(id_sort[i].p, id_sort[j].p))
+    {
+      index_i = id_sort[i].index;
+      index_j = id_sort[j].index;
+      if (index_j > index_i)
+      {
+        index = index_j;
+      }
+      else
+      {
+        index = index_i;
+        i = j;
+      }
+      pDelete(&id->m[index]);
+    }
+    else
+    {
+      i = j;
+    }
+  }
+  omFreeSize((ADDRESS)(id_sort), idsize*sizeof(poly_sort));
+}
+
 /*2
 *sorts the ideal w.r.t. the actual ringordering
 *uses lex-ordering when nolex = FALSE
@@ -1583,6 +1670,9 @@ ideal idSyzygies (ideal  h1, tHomog h,intvec **w, BOOLEAN setSyzComp,
   && (setRegularity)
   && (h==isHomog)
   && (!rIsPluralRing(currRing))
+  #ifdef HAVE_RINGS
+  && (!rField_is_Ring(currRing))
+  #endif
   )
   {
     ring dp_C_ring = rCurrRingAssure_dp_C();
@@ -1698,7 +1788,7 @@ ideal idLiftStd (ideal  h1, matrix* ma, tHomog hi, ideal * syz)
 
   k=si_max(1,(int)idRankFreeModule(h1));
 
-  if ((k==1) && (!lift3)) verbose |=Sy_bit(V_IDLIFT);
+  if (!lift3) verbose |=Sy_bit(V_IDLIFT);
 
   ring orig_ring = currRing;
   ring syz_ring = rCurrRingAssure_SyzComp();
@@ -2977,6 +3067,7 @@ void idCompactify(ideal id)
   }
   else
   {
+    if (IDELEMS(id)>10) idDelEquals(id);
     idDelMultiples(id);
   }
   idSkipZeroes(id);
@@ -4062,11 +4153,11 @@ BOOLEAN idIsZeroDim(ideal I)
   return res;
 }
 
-void idNormalize(ideal I)
+void idNormalize(ideal I) /* for ideal/matrix */
 {
   if (rField_has_simple_inverse()) return; /* Z/p, GF(p,n), R, long R/C */
   int i;
-  for(i=IDELEMS(I)-1;i>=0;i--)
+  for(i=I->nrows*I->ncols-1;i>=0;i--)
   {
     pNormalize(I->m[i]);
   }
@@ -4124,7 +4215,7 @@ poly p_ChineseRemainder(poly *xx, number *x,number *q, int rl, const ring R)
     for(j=rl-1;j>=0;j--)
     {
       hh=xx[j];
-      if ((hh!=NULL) && (p_LmCmp(r,hh,R)==0))
+      if ((hh!=NULL) && (p_LmCmp(h,hh,R)==0))
       {
         x[j]=pGetCoeff(hh);
         hh=p_LmFreeAndNext(hh,R);
@@ -4138,14 +4229,18 @@ poly p_ChineseRemainder(poly *xx, number *x,number *q, int rl, const ring R)
     {
       x[j]=NULL; // nlInit(0...) takes no memory
     }
-    if (n_IsZero(n,R)) p_Delete(&h,R);
+    assume(pNext(h)==NULL);
+    if (n_IsZero(n,R)) p_LmDelete(&h,R);
     else
     {
-      p_SetCoeff(h,n,R);
       //Print("new mon:");pWrite(h);
-      res_p=p_Add_q(res_p,h,R);
+      p_SetCoeff(h,n,R);
+      pNext(h)=res_p;
+      res_p=h; // building res_p in reverse order!
     }
   }
+  res_p=pReverse(res_p);
+  p_Test(res_p,R);
   return res_p;
 }
 ideal idChineseRemainder(ideal *xx, number *q, int rl)
@@ -4172,7 +4267,7 @@ ideal idChineseRemainder(ideal *xx, number *q, int rl)
   omFreeSize(p,rl*sizeof(poly));
   omFreeSize(x,rl*sizeof(number));
   for(i=rl-1;i>=0;i--) idDelete(&(xx[i]));
-  omFree(xx);
+  omFreeSize(xx,rl*sizeof(ideal));
   return result;
 }
 #endif
