@@ -432,11 +432,7 @@ KINLINE void  sTObject::pCleardenom()
 	}
       else
 	{
-#ifdef HAVE_RATGRING
 	  p_Cleardenom_n(p, currRing, n);
-#else
-	  p_Cleardenom_n(p, currRing, n);
-#endif
 	}
       if (!nIsOne(n))
 	{
@@ -456,11 +452,7 @@ KINLINE void  sTObject::pCleardenom()
 	}
       else
 	{
-#ifdef HAVE_RATGRING
 	  p_Cleardenom(p, currRing);
-#else
-	  p_Cleardenom(p, currRing);
-#endif
 	}
     }
 }
@@ -1012,7 +1004,7 @@ KINLINE void k_GetStrongLeadTerms(const poly p1, const poly p2, const ring leadR
   m2 = p_Init(tailRing);
   lcm = p_Init(leadRing);
 
-  for (i = leadRing->N; i>=1; i--)
+  for (i = leadRing->N; i>=0; i--)
   {
     e1 = p_GetExp(p1,i,leadRing);
     e2 = p_GetExp(p2,i,leadRing);
@@ -1023,23 +1015,16 @@ KINLINE void k_GetStrongLeadTerms(const poly p1, const poly p2, const ring leadR
       //p_SetExp(m1,i,0, tailRing); // done by p_Init
       s = e1;
     }
-    else
+    else if (x < 0)
     {
       p_SetExp(m1,i,-x, tailRing);
       //p_SetExp(m2,i,0, tailRing); // done by p_Init
       s = e2;
     }
+    else
+      s = e1; // e1==e2
     p_SetExp(lcm,i,s, leadRing);
   }
-  if ((s=pGetComp(p1))!=0)
-  {
-    p_SetComp(lcm,s, leadRing);
-  } 
-  else if ((s=pGetComp(p2))!=0)
-  {
-    p_SetComp(lcm,s, leadRing);
-  } 
-  // else p_SetComp(lcm,0,tailRing); // done by p_Init
 
   p_Setm(m1, tailRing);
   p_Setm(m2, tailRing);
