@@ -7,8 +7,6 @@
 
 // TODO: why the following is here instead of mod2.h???
 
-// define if tailrings should be used
-#define HAVE_TAIL_RING
 
 // define if buckets should be used
 #define MORA_USE_BUCKETS
@@ -1172,9 +1170,6 @@ ideal mora (ideal F, ideal Q,intvec *w,intvec *hilb,kStrategy strat)
 #ifdef HAVE_ASSUME
   mora_count++;
   mora_loop_count = 0;
-#endif
-#ifdef KDEBUG
-  om_Opts.MinTrack = 5;
 #endif
   int srmax;
   int lrmax = 0;
@@ -1511,13 +1506,15 @@ poly kNF1 (ideal F,ideal Q,poly q, kStrategy strat, int lazyReduce)
   }
   /*- release temp data------------------------------- -*/
   cleanT(strat);
+  assume(strat->L==NULL); /*strat->L unsed */
+  assume(strat->B==NULL); /*strat->B unused */
   omFreeSize((ADDRESS)strat->T,strat->tmax*sizeof(TObject));
   omFreeSize((ADDRESS)strat->ecartS,IDELEMS(strat->Shdl)*sizeof(int));
   omFreeSize((ADDRESS)strat->sevS,IDELEMS(strat->Shdl)*sizeof(unsigned long));
   omFreeSize((ADDRESS)strat->NotUsedAxis,(pVariables+1)*sizeof(BOOLEAN));
-  omfree(strat->sevT);
-  omfree(strat->S_2_R);
-  omfree(strat->R);
+  omFree(strat->sevT);
+  omFree(strat->S_2_R);
+  omFree(strat->R);
 
   if ((Q!=NULL)&&(strat->fromQ!=NULL))
   {
@@ -1650,13 +1647,15 @@ ideal kNF1 (ideal F,ideal Q,ideal q, kStrategy strat, int lazyReduce)
     //  res->m[i]=NULL;
   }
   /*- release temp data------------------------------- -*/
+  assume(strat->L==NULL); /*strat->L unsed */
+  assume(strat->B==NULL); /*strat->B unused */
   omFreeSize((ADDRESS)strat->T,strat->tmax*sizeof(TObject));
   omFreeSize((ADDRESS)strat->ecartS,IDELEMS(strat->Shdl)*sizeof(int));
   omFreeSize((ADDRESS)strat->sevS,IDELEMS(strat->Shdl)*sizeof(unsigned long));
   omFreeSize((ADDRESS)strat->NotUsedAxis,(pVariables+1)*sizeof(BOOLEAN));
-  omfree(strat->sevT);
-  omfree(strat->S_2_R);
-  omfree(strat->R);
+  omFree(strat->sevT);
+  omFree(strat->S_2_R);
+  omFree(strat->R);
   if ((Q!=NULL)&&(strat->fromQ!=NULL))
   {
     i=((IDELEMS(Q)+IDELEMS(F)+15)/16)*16;
@@ -2402,68 +2401,73 @@ ideal kInterRedBba (ideal F, ideal Q, int &need_retry)
 #endif
 
       // enter into S, L, and T
-      //if ((!TEST_OPT_IDLIFT) || (pGetComp(strat->P.p) <= strat->syzComp))
+      if ((!TEST_OPT_IDLIFT) || (pGetComp(strat->P.p) <= strat->syzComp))
+      {
         enterT(strat->P, strat);
-      // posInS only depends on the leading term
-      strat->enterS(strat->P, pos, strat, strat->tl);
-
+        // posInS only depends on the leading term
+        strat->enterS(strat->P, pos, strat, strat->tl);
+          if (strat->sl>srmax) srmax = strat->sl;
+          if (pos<strat->sl)
+          {
+            need_retry++;
+            // move all "larger" elements fromS to L
+            // remove them from T
+            int ii=pos+1;
+            for(;ii<=strat->sl;ii++)
+            {
+              LObject h;
+              memset(&h,0,sizeof(h));
+              h.tailRing=strat->tailRing;
+              h.p=strat->S[ii]; strat->S[ii]=NULL;
+              strat->initEcart(&h);
+              h.sev=strat->sevS[ii];
+              int jj=strat->tl;
+              while (jj>=0)
+              {
+                if (strat->T[jj].p==h.p)
+                {
+                  strat->T[jj].p=NULL;
+                  if (jj<strat->tl)
+                  {
+                    memmove(&(strat->T[jj]),&(strat->T[jj+1]),
+                            (strat->tl-jj)*sizeof(strat->T[jj]));
+                    memmove(&(strat->sevT[jj]),&(strat->sevT[jj+1]),
+                            (strat->tl-jj)*sizeof(strat->sevT[jj]));
+                  }
+                  strat->tl--;
+                  break;
+                }
+                jj--;
+              }
+              int lpos=strat->posInL(strat->L,strat->Ll,&h,strat);
+              enterL(&strat->L,&strat->Ll,&strat->Lmax,h,lpos);
+              #ifdef KDEBUG
+              if (TEST_OPT_DEBUG)
+              {
+                Print("move S[%d] -> L[%d]: ",ii,pos);
+                p_wrp(h.p,currRing, strat->tailRing);
+                PrintLn();
+              }
+              #endif
+            }
+            if (strat->fromQ!=NULL)
+            {
+              for(ii=pos+1;ii<=strat->sl;ii++) strat->fromQ[ii]=0;
+            }
+            strat->sl=pos;
+          }
+        }
+      }
+      else
+      {
+        // clean P
+      }
       if (strat->P.lcm!=NULL)
 #ifdef HAVE_RINGS
         pLmDelete(strat->P.lcm);
 #else
         pLmFree(strat->P.lcm);
 #endif
-      if (strat->sl>srmax) srmax = strat->sl;
-      if (pos<strat->sl)
-      {
-        need_retry++;
-        // move all "larger" elements fromS to L
-        // remove them from T
-        int ii=pos+1;
-        for(;ii<=strat->sl;ii++)
-        {
-          LObject h;
-          memset(&h,0,sizeof(h));
-          h.tailRing=strat->tailRing;
-          h.p=strat->S[ii]; strat->S[ii]=NULL;
-          strat->initEcart(&h);
-          h.sev=strat->sevS[ii];
-          int jj=strat->tl;
-          while (jj>=0)
-          {
-            if (strat->T[jj].p==h.p)
-            {
-              strat->T[jj].p=NULL;
-              if (jj<strat->tl)
-              {
-                memmove(&(strat->T[jj]),&(strat->T[jj+1]),
-                        (strat->tl-jj)*sizeof(strat->T[jj]));
-                memmove(&(strat->sevT[jj]),&(strat->sevT[jj+1]),
-                        (strat->tl-jj)*sizeof(strat->sevT[jj]));
-              }
-              strat->tl--;
-              break;
-            }
-            jj--;
-          }
-          int lpos=strat->posInL(strat->L,strat->Ll,&h,strat);
-          enterL(&strat->L,&strat->Ll,&strat->Lmax,h,lpos);
-          #ifdef KDEBUG
-          if (TEST_OPT_DEBUG)
-          {
-            Print("move S[%d] -> L[%d]: ",ii,pos);
-            p_wrp(h.p,currRing, strat->tailRing);
-            PrintLn();
-          }
-          #endif
-        }
-        if (strat->fromQ!=NULL)
-        {
-          for(ii=pos+1;ii<=strat->sl;ii++) strat->fromQ[ii]=0;
-        }
-        strat->sl=pos;
-      }
-    }
 
 #ifdef KDEBUG
     if (TEST_OPT_DEBUG)
