@@ -7,11 +7,6 @@
 
 // #define PDEBUG 2
 
-// TODO: why the following is here instead of mod2.h???
-
-// define to enable tailRings
-#define HAVE_TAIL_RING
-
 #include <kernel/mod2.h>
 
 #ifndef NDEBUG
@@ -973,7 +968,6 @@ ideal bba (ideal F, ideal Q,intvec *w,intvec *hilb,kStrategy strat)
   bba_count++;
   int loop_count = 0;
 #endif /* KDEBUG */
-  om_Opts.MinTrack = 5;
   int   srmax,lrmax, red_result = 1;
   int   olddeg,reduc;
   int hilbeledeg=1,hilbcount=0,minimcnt=0;
@@ -1185,38 +1179,44 @@ ideal bba (ideal F, ideal Q,intvec *w,intvec *hilb,kStrategy strat)
       }
 
       // enter into S, L, and T
-      //if ((!TEST_OPT_IDLIFT) || (pGetComp(strat->P.p) <= strat->syzComp))
+      if ((!TEST_OPT_IDLIFT) || (pGetComp(strat->P.p) <= strat->syzComp))
+      {
         enterT(strat->P, strat);
 #ifdef HAVE_RINGS
-      if (rField_is_Ring(currRing))
-        superenterpairs(strat->P.p,strat->sl,strat->P.ecart,pos,strat, strat->tl);
-      else
+        if (rField_is_Ring(currRing))
+          superenterpairs(strat->P.p,strat->sl,strat->P.ecart,pos,strat, strat->tl);
+        else
 #endif
-        enterpairs(strat->P.p,strat->sl,strat->P.ecart,pos,strat, strat->tl);
-      // posInS only depends on the leading term
-      strat->enterS(strat->P, pos, strat, strat->tl);
+          enterpairs(strat->P.p,strat->sl,strat->P.ecart,pos,strat, strat->tl);
+        // posInS only depends on the leading term
+        strat->enterS(strat->P, pos, strat, strat->tl);
 #if 0
-      int pl=pLength(strat->P.p);
-      if (pl==1)
-      {
-        //if (TEST_OPT_PROT)
-        //PrintS("<1>");
-      }
-      else if (pl==2)
-      {
-        //if (TEST_OPT_PROT)
-        //PrintS("<2>");
-      }
+        int pl=pLength(strat->P.p);
+        if (pl==1)
+        {
+          //if (TEST_OPT_PROT)
+          //PrintS("<1>");
+        }
+        else if (pl==2)
+        {
+          //if (TEST_OPT_PROT)
+          //PrintS("<2>");
+        }
 #endif
-      if (hilb!=NULL) khCheck(Q,w,hilb,hilbeledeg,hilbcount,strat);
+        if (hilb!=NULL) khCheck(Q,w,hilb,hilbeledeg,hilbcount,strat);
 //      Print("[%d]",hilbeledeg);
+        if (strat->sl>srmax) srmax = strat->sl;
+      }
+      else
+      {
+        // clean P
+      }
       if (strat->P.lcm!=NULL)
 #ifdef HAVE_RINGS
         pLmDelete(strat->P.lcm);
 #else
         pLmFree(strat->P.lcm);
 #endif
-      if (strat->sl>srmax) srmax = strat->sl;
     }
     else if (strat->P.p1 == NULL && strat->minim > 0)
     {
@@ -1351,14 +1351,14 @@ poly kNF2 (ideal F,ideal Q,poly q,kStrategy strat, int lazyReduce)
     }
   }
   /*- release temp data------------------------------- -*/
-  omfree(strat->sevS);
-  omfree(strat->ecartS);
-  omfree(strat->T);
-  omfree(strat->sevT);
-  omfree(strat->R);
+  assume(strat->L==NULL); /* strat->L unused */
+  assume(strat->B==NULL); /* strat->B unused */
+  omFree(strat->sevS);
+  omFree(strat->ecartS);
+  assume(strat->T==NULL);//omfree(strat->T);
+  assume(strat->sevT==NULL);//omfree(strat->sevT);
+  assume(strat->R==NULL);//omfree(strat->R);
   omfree(strat->S_2_R);
-  omfree(strat->L);
-  omfree(strat->B);
   omfree(strat->fromQ);
   idDelete(&strat->Shdl);
   test=save_test;
@@ -1427,14 +1427,14 @@ ideal kNF2 (ideal F,ideal Q,ideal q,kStrategy strat, int lazyReduce)
     //  res->m[i]=NULL;
   }
   /*- release temp data------------------------------- -*/
-  omfree(strat->sevS);
-  omfree(strat->ecartS);
-  omfree(strat->T);
-  omfree(strat->sevT);
-  omfree(strat->R);
+  assume(strat->L==NULL); /* strat->L unused */
+  assume(strat->B==NULL); /* strat->B unused */
+  omFree(strat->sevS);
+  omFree(strat->ecartS);
+  assume(strat->T==NULL);//omfree(strat->T);
+  assume(strat->sevT==NULL);//omfree(strat->sevT);
+  assume(strat->R==NULL);//omfree(strat->R);
   omfree(strat->S_2_R);
-  omfree(strat->L);
-  omfree(strat->B);
   omfree(strat->fromQ);
   idDelete(&strat->Shdl);
   test=save_test;
@@ -1452,7 +1452,6 @@ ideal bbaShift(ideal F, ideal Q,intvec *w,intvec *hilb,kStrategy strat, int upto
   bba_count++;
   int loop_count = 0;
 #endif
-  om_Opts.MinTrack = 5;
   int   srmax,lrmax, red_result = 1;
   int   olddeg,reduc;
   int hilbeledeg=1,hilbcount=0,minimcnt=0;
