@@ -17,6 +17,9 @@
 #include <kernel/structs.h>
 #include <kernel/pShallowCopyDelete.h>
 
+// define if tailrings should be used
+#define HAVE_TAIL_RING
+
 #if 1
 #define setmax 16
 #define setmaxL ((4096-12)/sizeof(LObject))
