@@ -55,7 +55,7 @@ static numberfunc
                 nacMult, nacSub, nacAdd, nacDiv, nacIntDiv;
 static number   (*nacGcd)(number a, number b, const ring r);
 static number   (*nacLcm)(number a, number b, const ring r);
-static number   (*nacInit)(int i, const ring r);
+static number   (*nacInit)(long i, const ring r);
 static int      (*nacInt)(number &n, const ring r);
 static void     (*nacDelete)(number *a, const ring r);
 #undef n_Delete
@@ -304,6 +304,27 @@ static napoly napInvers(napoly x, const napoly c)
     }
     return x;
   }
+  #ifdef HAVE_FACTORY
+  {
+    number den=p_GetAllDenom(x,nacRing);
+    poly xp=p_Mult_nn(p_Copy(x,nacRing),den,nacRing);
+    poly xp_f, mipo_f,g;
+    singclap_extgcd_r(xp,naMinimalPoly,g,xp_f,mipo_f,nacRing);
+    BOOLEAN zero_div_found=(!p_IsConstant(g,nacRing));
+    p_Delete(&g,nacRing);
+    p_Delete(&xp,nacRing);
+    p_Delete(&mipo_f,nacRing);
+    if (zero_div_found)
+    {
+      p_Delete(&xp,nacRing);
+      n_Delete(&den,nacRing);
+      goto zero_divisor;
+    }
+    xp_f=p_Mult_nn(xp_f,den,nacRing);
+    n_Delete(&den,nacRing);
+    return xp_f;
+  }
+  #else
   y = napCopy(c);
   napDivMod(y, x, &qa, &r);
   if (r==NULL)
@@ -380,6 +401,7 @@ static napoly napInvers(napoly x, const napoly c)
     q = napAdd(q, qa);
     qa = y;
   }
+  #endif
 // zero divisor found:
 zero_divisor:
   Werror("zero divisor found - your minpoly is not irreducible");
@@ -897,7 +919,7 @@ napoly napTailred (napoly q)
 /*2
 *  z:= i
 */
-number naInit(int i, const ring r)
+number naInit(long i, const ring r)
 {
   if (i!=0)
   {
@@ -986,8 +1008,9 @@ void naDelete(number *p, const ring r)
 */
 number naCopy(number p)
 {
+  // no naTest: naCopy may be applied to "non-conforming" data
+  // while mapping from rational function to alg. ext. etc.
   if (p==NULL) return NULL;
-  naTest(p);
   lnumber erg;
   lnumber src = (lnumber)p;
   erg = (lnumber)omAlloc0Bin(rnumber_bin);
@@ -2370,6 +2393,20 @@ nMapFunc naSetMap(const ring src, const ring dst)
         return naCopy;    /* Q(a) -> Q(a) */
       return naMapQaQb;   /* Q(a..) -> Q(a..) */
     }
+    if (rField_is_Zp_a(src))   /* Z/p(a) -> Q(a) */
+    {
+      int i;
+      naParsToCopy=0;
+      for(i=0;i<rPar(src);i++)
+      {
+        if ((i>=rPar(dst))
+        ||(strcmp(src->parameter[i],dst->parameter[i])!=0))
+           return NULL;
+        naParsToCopy++;
+      }
+      nacMap = nlSetMap(src->algring, dst->algring);
+      return naMapQaQb;
+    }
   }
   /*-----------------------------------------------------*/
   if (rField_is_Zp_a(dst)) /* -> Z/p(a) */
@@ -2413,6 +2450,21 @@ nMapFunc naSetMap(const ring src, const ring dst)
         return naCopy;    /* Z/p(a) -> Z/p(a) */
       return naMapQaQb;   /* Z/p(a),Z/p'(a) -> Z/p(b)*/
     }
+    if (rField_is_Q_a(src))
+    {
+      int i;
+      naParsToCopy=0;
+      for(i=0;i<rPar(src);i++)
+      {
+        if ((i>=rPar(dst))
+        ||(strcmp(src->parameter[i],dst->parameter[i])!=0))
+           return NULL;
+        naParsToCopy++;
+      }
+      npPrimeM = dst->algring->ch;
+      nacMap = npSetMap(src->algring, dst->algring);
+      return naMapQaQb;
+    }
   }
   return NULL;      /* default */
 }
@@ -2712,23 +2764,39 @@ poly p_ChineseRemainder(poly *xx, number *x,number *q, int rl, const ring R);
 number naChineseRemainder(number *x, number *q,int rl, const ring r)
 {
   poly *xx=(poly*)omAlloc0(rl*sizeof(poly));
+  poly *yy=(poly*)omAlloc0(rl*sizeof(poly));
   number *qq=(number*)omAlloc0(rl*sizeof(number));
   number *temp=(number*)omAlloc0(rl*sizeof(number));
   lnumber t;
-  for(int i=rl-1;i>=0;i--)
+  int i;
+  for(i=rl-1;i>=0;i--)
   {
     if(x[i]!=NULL)
     {
       t=(lnumber)x[i];
       xx[i]=t->z;
+      if (t->n==NULL) yy[i]=p_One(r->algring);
+      else            yy[i]=p_Copy(t->n,r->algring);
     }
     t=(lnumber)q[i];
     qq[i]=pGetCoeff(t->z);
   }
+  number *qq2=(number*)omAlloc0(rl*sizeof(number));
+  memcpy(qq2,qq,rl*sizeof(number));
+  number *temp2=(number*)omAlloc0(rl*sizeof(number));
+  memcpy(temp2,temp,rl*sizeof(number));
   poly rr=p_ChineseRemainder(xx,temp,qq,rl,r->algring);
   t=ALLOC0_LNUMBER();
   t->z=rr;
-  t->s=2;
+  //t->s=0;
+  rr=p_ChineseRemainder(yy,temp2,qq2,rl,r->algring);
+  if ((rr!=NULL) && (pNext(rr)==NULL)
+  && (n_IsOne(pGetCoeff(rr),r->algring)))
+  {
+    p_Delete(&rr,r->algring);
+    t->s=2;
+  }
+  t->n=rr;
   return (number)t;
 }
 
@@ -2737,14 +2805,14 @@ number naFarey(number nN, number nP, const ring r)
 {
   lnumber N = (lnumber)nN;
   /* nP is a BIGINT_CMD */
-  if (N->n!=NULL)
-  {
-    WerrorS("rational function not allowed in naFarey");
-    return NULL;
-  }
   if (N==NULL) return NULL;
   lnumber rr=ALLOC0_LNUMBER();
   rr->z=p_Farey(N->z,nP,r->algring);
-  rr->s = 2;
+  if (N->n==NULL)
+    rr->s = 2;
+  else
+  {
+    rr->n=p_Farey(N->n,nP,r->algring);
+  }
   return (number)rr;
 }
