@@ -9,8 +9,8 @@
 #include <kernel/structs.h>
 #include <kernel/longrat.h>
 
+#if SIZEOF_LONG == 8
 // 64 bit version:
-#if SIZE_OF_LONG == 8
 #define MAX_NUM_SIZE 60
 #define POW_2_28 (1L<<60)
 #define LONG long
@@ -124,9 +124,7 @@ number nlShort3_noinline(number x) // assume x->s==3
   return nlShort3(x);
 }
 
-//#ifndef SI_THREADS
 omBin rnumber_bin = omGetSpecBin(sizeof(snumber));
-//#endif
 
 number nlOne=INT_TO_SR(1);
 
@@ -501,7 +499,18 @@ int nlInt(number &i, const ring r)
 {
   nlTest(i);
   nlNormalize(i);
-  if (SR_HDL(i) &SR_INT) return SR_TO_INT(i);
+  if (SR_HDL(i) &SR_INT)
+  {
+    #if SIZEOF_LONG == 8
+    int dummy = SR_TO_INT(i);
+    if((long)dummy == SR_TO_INT(i))
+      return dummy /*SR_TO_INT(i)*/;
+    else
+      return 0;
+    #else
+    return SR_TO_INT(i);
+    #endif
+  }
   if (i->s==3)
   {
     if(mpz_size1(i->z)>MP_SMALL) return 0;
@@ -625,46 +634,38 @@ number nlInvers(number a)
 */
 number   nlExactDiv(number a, number b)
 {
-  if (b==INT_TO_SR(0))
+  mpz_t a_r;
+  mpz_t b_r;
+  mpz_ptr aa=a_r;
+  mpz_ptr bb=b_r;
+  if (SR_HDL(a) & SR_INT)
   {
-    WerrorS(nDivBy0);
-    return INT_TO_SR(0);
+    mpz_init_set_si(a_r,SR_TO_INT(a));
   }
-  if (a==INT_TO_SR(0))
-    return INT_TO_SR(0);
-  number u;
-  if (SR_HDL(a) & SR_HDL(b) & SR_INT)
-  {
-    /* the small int -(1<<28) divided by -1 is the large int (1<<28)   */
-    if ((a==INT_TO_SR(-(POW_2_28)))&&(b==INT_TO_SR(-1L)))
-    {
-      return nlRInit(POW_2_28);
-    }
-    long aa=SR_TO_INT(a);
-    long bb=SR_TO_INT(b);
-    return INT_TO_SR(aa/bb);
-  }
-  number bb=NULL;
+  else
+    aa=a->z;
   if (SR_HDL(b) & SR_INT)
   {
-    bb=nlRInit(SR_TO_INT(b));
-    b=bb;
+    mpz_init_set_si(b_r,SR_TO_INT(b));
   }
-  u=ALLOC_RNUMBER();
+  else
+    bb=b->z;
+  number u=ALLOC_RNUMBER();
 #if defined(LDEBUG)
   u->debug=123456;
 #endif
-  mpz_init(u->z);
-  /* u=a/b */
   u->s = 3;
-  MPZ_EXACTDIV(u->z,a->z,b->z);
-  if (bb!=NULL)
+  assume(a->s==3);
+  assume(b->s==3);
+  mpz_init(u->z);
+  MPZ_EXACTDIV(u->z,aa,bb);
+  if (SR_HDL(a) & SR_INT)
   {
-    mpz_clear(bb->z);
-#if defined(LDEBUG)
-    bb->debug=654324;
-#endif
-    FREE_RNUMBER(bb);
+    mpz_clear(aa);
+  }
+  if (SR_HDL(b) & SR_INT)
+  {
+    mpz_clear(bb);
   }
   u=nlShort3(u);
   nlTest(u);
@@ -683,54 +684,39 @@ number nlIntDiv (number a, number b)
   }
   if (a==INT_TO_SR(0))
     return INT_TO_SR(0);
-  number u;
-  if (SR_HDL(a) & SR_HDL(b) & SR_INT)
-  {
-    /* the small int -(1<<28) divided by -1 is the large int (1<<28)   */
-    if ((a==INT_TO_SR(-(POW_2_28)))&&(b==INT_TO_SR(-1L)))
-    {
-      return nlRInit(POW_2_28);
-    }
-    long aa=SR_TO_INT(a);
-    long bb=SR_TO_INT(b);
-    return INT_TO_SR(aa/bb);
-  }
+  mpz_t a_r;
+  mpz_t b_r;
+  mpz_ptr aa=a_r;
+  mpz_ptr bb=b_r;
   if (SR_HDL(a) & SR_INT)
   {
-    /* the small int -(1<<28) divided by 2^28 is 1   */
-    if (a==INT_TO_SR(-(POW_2_28)))
-    {
-      if(mpz_cmp_si(b->z,(POW_2_28))==0)
-      {
-        return INT_TO_SR(-1);
-      }
-    }
-    /* a is a small and b is a large int: -> 0 */
-    return INT_TO_SR(0);
+    mpz_init_set_si(a_r,SR_TO_INT(a));
   }
-  number bb=NULL;
+  else
+    aa=a->z;
   if (SR_HDL(b) & SR_INT)
   {
-    bb=nlRInit(SR_TO_INT(b));
-    b=bb;
+    mpz_init_set_si(b_r,SR_TO_INT(b));
   }
-  u=ALLOC_RNUMBER();
+  else
+    bb=b->z;
+  number u=ALLOC_RNUMBER();
 #if defined(LDEBUG)
   u->debug=123456;
 #endif
   assume(a->s==3);
   assume(b->s==3);
-  mpz_init_set(u->z,a->z);
+  mpz_init_set(u->z,aa);
   /* u=u/b */
   u->s = 3;
-  MPZ_DIV(u->z,u->z,b->z);
-  if (bb!=NULL)
+  MPZ_DIV(u->z,u->z,bb);
+  if (SR_HDL(a) & SR_INT)
   {
-    mpz_clear(bb->z);
-#if defined(LDEBUG)
-    bb->debug=654324;
-#endif
-    FREE_RNUMBER(bb);
+    mpz_clear(aa);
+  }
+  if (SR_HDL(b) & SR_INT)
+  {
+    mpz_clear(bb);
   }
   u=nlShort3(u);
   nlTest(u);
@@ -749,45 +735,36 @@ number nlIntMod (number a, number b)
   }
   if (a==INT_TO_SR(0))
     return INT_TO_SR(0);
-  number u;
-  if (SR_HDL(a) & SR_HDL(b) & SR_INT)
-  {
-    LONG bb=SR_TO_INT(b);
-    LONG c=SR_TO_INT(a)%bb;
-    return INT_TO_SR(c);
-  }
+  mpz_t a_r;
+  mpz_t b_r;
+  mpz_ptr aa=a_r;
+  mpz_ptr bb=b_r;
   if (SR_HDL(a) & SR_INT)
   {
-    /* a is a small and b is a large int: -> a */
-    return a;
+    mpz_init_set_si(a_r,SR_TO_INT(a));
   }
-  number bb=NULL;
+  else
+    aa=a->z;
   if (SR_HDL(b) & SR_INT)
   {
-    bb=nlRInit(SR_TO_INT(b));
-    b=bb;
+    mpz_init_set_si(b_r,SR_TO_INT(b));
   }
-  u=ALLOC_RNUMBER();
+  else
+    bb=b->z;
+  number u=ALLOC_RNUMBER();
 #if defined(LDEBUG)
   u->debug=123456;
 #endif
-  mpz_init(u->z);
   u->s = 3;
-  mpz_mod(u->z,a->z,b->z);
-  if (bb!=NULL)
+  mpz_init(u->z);
+  mpz_mod(u->z,aa,bb);
+  if (SR_HDL(a) & SR_INT)
   {
-    mpz_clear(bb->z);
-#if defined(LDEBUG)
-    bb->debug=654324;
-#endif
-    FREE_RNUMBER(bb);
+    mpz_clear(a_r);
   }
-  if (mpz_isNeg(u->z))
+  if (SR_HDL(b) & SR_INT)
   {
-    if (mpz_isNeg(b->z))
-      mpz_sub(u->z,u->z,b->z);
-    else
-      mpz_add(u->z,u->z,b->z);
+    mpz_clear(b_r);
   }
   u=nlShort3(u);
   nlTest(u);
@@ -799,38 +776,45 @@ number nlIntMod (number a, number b)
 */
 number nlDiv (number a, number b)
 {
-  number u;
   if (nlIsZero(b))
   {
     WerrorS(nDivBy0);
     return INT_TO_SR(0);
   }
-  u=ALLOC_RNUMBER();
-  u->s=0;
-#if defined(LDEBUG)
-  u->debug=123456;
-#endif
+  number u;
 // ---------- short / short ------------------------------------
   if (SR_HDL(a) & SR_HDL(b) & SR_INT)
   {
     long i=SR_TO_INT(a);
     long j=SR_TO_INT(b);
+    if (j==1L)
+    {
+      return a;
+    }
     if ((i==-POW_2_28) && (j== -1L))
     {
-      FREE_RNUMBER(u);
       return nlRInit(POW_2_28);
     }
     long r=i%j;
     if (r==0)
     {
-      FREE_RNUMBER(u);
       return INT_TO_SR(i/j);
     }
-    mpz_init_set_si(u->z,(long)i);
-    mpz_init_set_si(u->n,(long)j);
+    u=ALLOC_RNUMBER();
+    u->s=0;
+    #if defined(LDEBUG)
+    u->debug=123456;
+    #endif
+    mpz_init_set_si(u->z,i);
+    mpz_init_set_si(u->n,j);
   }
   else
   {
+    u=ALLOC_RNUMBER();
+    u->s=0;
+    #if defined(LDEBUG)
+    u->debug=123456;
+    #endif
     mpz_init(u->z);
 // ---------- short / long ------------------------------------
     if (SR_HDL(a) & SR_INT)
@@ -1116,13 +1100,8 @@ void nlNormalize (number &x)
       x->s=1;
       if (mpz_cmp_si(gcd,(long)1)!=0)
       {
-        mpz_t r;
-        mpz_init(r);
-        MPZ_EXACTDIV(r,x->z,gcd);
-        mpz_set(x->z,r);
-        MPZ_EXACTDIV(r,x->n,gcd);
-        mpz_set(x->n,r);
-        mpz_clear(r);
+        MPZ_EXACTDIV(x->z,x->z,gcd);
+        MPZ_EXACTDIV(x->n,x->n,gcd);
         if (mpz_cmp_si(x->n,(long)1)==0)
         {
           mpz_clear(x->n);
@@ -2056,14 +2035,14 @@ number nlRInit (long i)
 /*2
 * z := i/j
 */
-number nlInit2 (int i, int j)
+number nlInit2 (long i, long j)
 {
   number z=ALLOC_RNUMBER();
 #if defined(LDEBUG)
   z->debug=123456;
 #endif
-  mpz_init_set_si(z->z,(long)i);
-  mpz_init_set_si(z->n,(long)j);
+  mpz_init_set_si(z->z,i);
+  mpz_init_set_si(z->n,j);
   z->s = 0;
   nlNormalize(z);
   return z;
@@ -2119,12 +2098,19 @@ LINLINE BOOLEAN nlEqual (number a, number b)
 }
 
 
-LINLINE number nlInit (int i, const ring r)
+LINLINE number nlInit (long i, const ring r)
 {
   number n;
-  LONG ii=(LONG)i;
-  if ( ((ii << 3) >> 3) == ii ) n=INT_TO_SR(ii);
-  else                          n=nlRInit(ii);
+  if ((i>=POW_2_28)||(i< -POW_2_28))
+  {
+     n=nlRInit(i);
+     n=nlShort3(n);
+  }
+  else
+  {
+    LONG ii=(LONG)i;
+    n=INT_TO_SR(ii);
+  }
   nlTest(n);
   return n;
 }
@@ -2198,7 +2184,7 @@ LINLINE number nlNeg (number a)
   nlTest(a);
   if(SR_HDL(a) &SR_INT)
   {
-    int r=SR_TO_INT(a);
+    long r=SR_TO_INT(a);
     if (r==(-(POW_2_28))) a=nlRInit(POW_2_28);
     else               a=INT_TO_SR(-r);
     return a;
@@ -2226,18 +2212,19 @@ LINLINE number nlAdd (number a, number b)
 number nlShort1(number a);
 number nlShort3_noinline(number x);
 
-LINLINE number nlInpAdd (number a, number b, const ring r)
+LINLINE void nlInpAdd (number &a, number b, const ring r)
 {
   // a=a+b
   if (SR_HDL(a) & SR_HDL(b) & SR_INT)
   {
     LONG r=SR_HDL(a)+SR_HDL(b)-1L;
     if ( ((r << 1) >> 1) == r )
-      return (number)(long)r;
+      a=(number)(long)r;
     else
-      return nlRInit(SR_TO_INT(r));
+      a=nlRInit(SR_TO_INT(r));
   }
-  return _nlInpAdd_aNoImm_OR_bNoImm(a,b);
+  else
+    a=_nlInpAdd_aNoImm_OR_bNoImm(a,b);
 }
 
 LINLINE number nlMult (number a, number b)
