@@ -770,7 +770,7 @@ void   mpMonomials(matrix c, int r, int var, matrix m)
       pDelete(&MATELEM(m,k,l));
     }
   }
-  omfreeSize((ADDRESS)m->m,MATROWS(m)*MATCOLS(m)*sizeof(poly));
+  omFreeSize((ADDRESS)m->m,MATROWS(m)*MATCOLS(m)*sizeof(poly));
   /* allocate monoms in the right size r x MATROWS(c)*/
   m->m=(polyset)omAlloc0(r*MATROWS(c)*sizeof(poly));
   MATROWS(m)=r;
@@ -1933,17 +1933,21 @@ char * iiStringMatrix(matrix im, int dim,char ch)
   int i,ii = MATROWS(im);
   int j,jj = MATCOLS(im);
   poly *pp = im->m;
-  char *s=StringSetS("");
+  StringSetS("");
+  char ch_s[2];
+  ch_s[0]=ch;
+  ch_s[1]='\0';
 
   for (i=0; i<ii; i++)
   {
     for (j=0; j<jj; j++)
     {
       pString0(*pp++);
-      s=StringAppend("%c",ch);
-      if (dim > 1) s = StringAppendS("\n");
+      StringAppendS(ch_s);
+      if (dim > 1) StringAppendS("\n");
     }
   }
+  char *s=StringEndS();
   s[strlen(s)- (dim > 1 ? 2 : 1)]='\0';
   return s;
 }
