@@ -356,13 +356,13 @@ PINLINE0 poly pReverse(poly p);
  *
  ***************************************************************/
 char*     p_String(poly p, ring lmRing, ring tailRing);
-char*     p_String0(poly p, ring lmRing, ring tailRing);
+void      p_String0(poly p, ring lmRing, ring tailRing);
 void      p_Write(poly p, ring lmRing, ring tailRing);
 void      p_Write0(poly p, ring lmRing, ring tailRing);
 void      p_wrp(poly p, ring lmRing, ring tailRing);
 
 PINLINE2 char*     p_String(poly p, ring p_ring);
-PINLINE2 char*     p_String0(poly p, ring p_ring);
+PINLINE2 void      p_String0(poly p, ring p_ring);
 PINLINE2 void      p_Write(poly p, ring p_ring);
 PINLINE2 void      p_Write0(poly p, ring p_ring);
 PINLINE2 void      p_wrp(poly p, ring p_ring);
@@ -440,9 +440,9 @@ BOOLEAN _pp_Test(poly p, ring lmRing, ring tailRing, int level);
 #define p_CheckRing(r)          ((void)0)
 #define P_CheckIf(cond, check)  ((void)0)
 
-#define p_Test(p,r)     (1)
-#define p_LmTest(p,r)   (1)
-#define pp_Test(p, lmRing, tailRing) (1)
+#define p_Test(p,r)     do {} while (0)
+#define p_LmTest(p,r)   do {} while (0)
+#define pp_Test(p, lmRing, tailRing) do {} while (0)
 
 #endif
 
