@@ -341,7 +341,7 @@ int pMaxCompProc(poly p);
 
 // let's inline those, so that we can call them from the debugger
 inline char*   pString(poly p)    {return p_String(p, currRing, currRing);}
-inline char*   pString0(poly p)   {return p_String0(p, currRing, currRing);}
+inline void    pString0(poly p)   {p_String0(p, currRing, currRing);}
 inline void    pWrite(poly p)     {p_Write(p, currRing, currRing);}
 inline void    pWrite0(poly p)    {p_Write0(p, currRing, currRing);}
 inline void    wrp(poly p)        {p_wrp(p, currRing, currRing);}
@@ -354,6 +354,7 @@ poly p_One(const ring r);
 #define   pOne()   p_One(currRing)
 
 void      p_Content(poly p, const ring r);
+poly p_Div_nn(poly p, const number n, const ring r);
 void      pSimpleContent(poly p, int s);
 poly      p_Cleardenom(poly p, const ring r);
 void      p_Cleardenom_n(poly p, const ring r,number &c);
@@ -422,8 +423,8 @@ BOOLEAN pComparePolys(poly p1,poly p2);
 
 #else // ! PDEBUG
 
-#define pTest(p)        ((void)0)
-#define pLmTest(p)      ((void)0)
+#define pTest(p)        do {} while (0)
+#define pLmTest(p)      do {} while (0)
 #endif
 
 #endif // POLYS_H
