@@ -491,15 +491,13 @@ void p_Content(poly ph, const ring r)
     if (ph!=NULL)
     {
       number k = nGetUnit(pGetCoeff(ph));
-      if (!nGreaterZero(pGetCoeff(ph))) k = nNeg(k); // in-place negation
       if (!nIsOne(k))
       {
         number tmpNumber = k;
         k = nInvers(k);
         nDelete(&tmpNumber);
-        poly h = pNext(ph);
-	p_Mult_nn(ph,k,currRing);
-	pNormalize(ph);
+        p_Mult_nn(ph,k,currRing);
+        pNormalize(ph);
       }
       nDelete(&k);
     }
@@ -959,6 +957,7 @@ poly p_Cleardenom(poly ph, const ring r)
   if (rField_is_Ring(r))
   {
     p_Content(ph,r);
+    if(!n_GreaterZero(pGetCoeff(ph),r)) ph = p_Neg(ph,r);
     return start;
   }
 #endif
@@ -1181,9 +1180,22 @@ number p_GetAllDenom(poly ph, const ring r)
     number h=n_GetDenom(pGetCoeff(p),r);
     if (!n_IsOne(h,r))
     {
-      number dd=n_Mult(d,h,r);
-      n_Delete(&d,r);
-      d=dd;
+      number common=n_Gcd(d,h,r);
+      if(n_IsOne(common,r))
+      {
+        number dd=n_Mult(d,h,r);
+        n_Delete(&d,r);
+        d=dd;
+      }
+      else
+      {
+        number dd=n_IntDiv(d,common,r);
+        //n_Normalize(dd,r);
+        n_Delete(&d,r);
+        d=n_Mult(dd,h,r);
+        n_Delete(&dd,r);
+      }
+      n_Delete(&common,r);
     }
     n_Delete(&h,r);
     pIter(p);
@@ -1237,6 +1249,12 @@ poly pPermPoly (poly p, int * perm, const ring oldRing, nMapFunc nMap,
       && ((rField_is_Zp_a()) || (rField_is_Q_a())))
       {
         nNormalize(n);
+        if ((n!=NULL) && (((lnumber)n)->z==NULL))
+        {
+          omFreeBin((ADDRESS)n,  rnumber_bin);
+          n=NULL;
+        }
+        nTest(n);
       }
       pGetCoeff(qq)=n;
     // coef may be zero:  pTest(qq);
