@@ -3,7 +3,7 @@
 ////////////////////////////////////////////////////////////
 ////////////////////////////////////////////////////////////
 // FACTORY - Includes
-#include <factory.h>
+#include <factory/factory.h>
 // Factor - Includes
 #include <tmpl_inst.h>
 #include <Factor.h>
