@@ -7,7 +7,7 @@
 #ifndef INCL_ALGFACTOR_H
 #define INCL_ALGFACTOR_H
 
-#include <factory.h>
+#include <factory/factory.h>
 #include <tmpl_inst.h>  // for typedef's
 
 CanonicalForm algcd(const CanonicalForm & f, const CanonicalForm & g, const CFList & as, const Varlist & order);
