@@ -4,7 +4,7 @@
 /*static const char * errmsg = "\nYou found a bug!\nPlease inform singular@mathematik.uni-kl.de\n Please include above information and your input (the ideal/polynomial and characteristic) in your bug-report.\nThank you.";*/
 ///////////////////////////////////////////////////////////////////////////////
 // FACTORY - Includes
-#include<factory.h>
+#include<factory/factory.h>
 #ifndef NOSTREAMIO
 #ifdef HAVE_IOSTREAM
 #include <iostream>
